@@ -41,6 +41,7 @@ extra_options = {
     'static': Option.simple,
     'reference': Option.simple,
     'release': Option.simple,
+    'release-with-assert': Option.simple,
     'release-with-debug-info': Option.simple,
     'asan': Option.simple,
     'mkl': Option.simple,
@@ -49,6 +50,7 @@ extra_options = {
     'thread-mpi': Option.bool,
     'gpu': Option.bool,
     'opencl': Option.bool,
+    'clang_cuda': Option.bool,
     'openmp': Option.bool,
     'nranks': Option.string,
     'npme': Option.string,
@@ -62,10 +64,13 @@ def do_build(context):
     cmake_opts['GMX_COMPILER_WARNINGS'] = 'ON'
     cmake_opts['GMX_DEFAULT_SUFFIX'] = 'OFF'
     cmake_opts['CMAKE_BUILD_TYPE'] = 'Debug'
+    cmake_opts['GMX_USE_RDTSCP'] = 'DETECT'
 
     if context.opts.reference:
         cmake_opts['CMAKE_BUILD_TYPE'] = 'Reference'
-    elif context.opts.release:
+    elif context.opts['release']:
+        cmake_opts['CMAKE_BUILD_TYPE'] = 'Release'
+    elif context.opts['release-with-assert']:
         cmake_opts['CMAKE_BUILD_TYPE'] = 'RelWithAssert'
     elif context.opts['release-with-debug-info']:
         cmake_opts['CMAKE_BUILD_TYPE'] = 'RelWithDebInfo'
@@ -95,7 +100,10 @@ def do_build(context):
             cmake_opts['GMX_USE_OPENCL'] = 'ON'
         else:
             cmake_opts['CUDA_TOOLKIT_ROOT_DIR'] = context.env.cuda_root
-            cmake_opts['CUDA_HOST_COMPILER'] = context.env.cuda_host_compiler
+            if context.opts.clang_cuda:
+                cmake_opts['GMX_CLANG_CUDA'] = 'ON'
+            else:
+                cmake_opts['CUDA_HOST_COMPILER'] = context.env.cuda_host_compiler
     else:
         cmake_opts['GMX_GPU'] = 'OFF'
     if context.opts.thread_mpi is False:
@@ -116,6 +124,12 @@ def do_build(context):
     if context.opts.x11:
         cmake_opts['GMX_X11'] = 'ON'
 
+    # At least hwloc on Jenkins produces a massive amount of reports about
+    # memory leaks, which cannot be reasonably suppressed because ASAN cannot
+    # produce a reasonable stack trace for them.
+    if context.opts.asan:
+        cmake_opts['GMX_HWLOC'] = 'OFF'
+
     regressiontests_path = context.workspace.get_project_dir(Project.REGRESSIONTESTS)
 
     if context.job_type == JobType.RELEASE:
@@ -159,7 +173,7 @@ def do_build(context):
     else:
         context.build_target(target='tests', keep_going=True)
 
-        context.run_ctest(args=['--output-on-failure'])
+        context.run_ctest(args=['--output-on-failure'], memcheck=context.opts.asan)
 
         context.build_target(target='install')
         # TODO: Consider what could be tested about the installed binaries.
@@ -171,8 +185,6 @@ def do_build(context):
             use_tmpi = not context.opts.mpi and context.opts.thread_mpi is not False
 
             cmd = 'perl gmxtest.pl -mpirun mpirun -xml -nosuffix all'
-            if context.opts.asan:
-                cmd+=' -parse asan_symbolize.py'
 
             # setting this stuff below is just a temporary solution,
             # it should all be passed as a proper the runconf from outside
@@ -201,4 +213,6 @@ def do_build(context):
                 cmd += ' -nt ' + nranks
             if context.opts.double:
                 cmd += ' -double'
+            if context.opts.asan:
+                context.env.set_env_var('ASAN_OPTIONS', 'detect_leaks=0')
             context.run_cmd(cmd, shell=True, failure_message='Regression tests failed to execute')
