@@ -51,6 +51,20 @@ Doxygen documentation file for directories in the source tree.
 \ingroup module_commandline
  */
 
+/*! \libinternal
+\dir src/gromacs/compat
+\brief \ref group_compatibility
+
+\ingroup group_compatibility
+ */
+
+/*! \internal
+\dir src/gromacs/compat/tests
+\brief Tests for \ref group_compatibility
+
+\ingroup group_compatibility
+ */
+
 /*! \libinternal
 \dir src/gromacs/onlinehelp
 \brief \ref module_onlinehelp
