@@ -1,8 +1,8 @@
-#!/usr/bin/python
+#!/usr/bin/env python2
 #
 # This file is part of the GROMACS molecular simulation package.
 #
-# Copyright (c) 2014,2015,2016, by the GROMACS development team, led by
+# Copyright (c) 2014,2015,2016,2017,2018, by the GROMACS development team, led by
 # Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
 # and including many others, as listed in the AUTHORS file in the
 # top-level source directory and at http://www.gromacs.org.
@@ -410,7 +410,7 @@ class Directory(object):
         self._module = None
         self._is_test_dir = False
         if parent and parent.is_test_directory() or \
-                self._name in ('tests', 'legacytests'):
+                self._name == 'tests':
             self._is_test_dir = True
         self._is_external = False
         if parent and parent.is_external() or self._name == 'external':
