@@ -41,18 +41,15 @@
 
 #include "gromacs/fileio/pdbio.h"
 #include "gromacs/fileio/confio.h"
-#include "symtab.h"
 #include "gromacs/utility/smalloc.h"
-#include "symtab.h"
+#include "gromacs/topology/symtab.h"
 #include "macros.h"
 #include "copyrite.h"
 #include "gromacs/commandline/pargs.h"
-#include "index.h"
-#include "vec.h"
+#include "gromacs/math/vec.h"
 #include "typedefs.h"
 #include "gbutil.h"
-#include "physics.h"
-#include "atomprop.h"
+#include "gromacs/math/units.h"
 
 void copy_atom(t_symtab *tab,t_atoms *a1,int i1,t_atoms *a2,int i2,
 	       rvec xin[],rvec xout[],rvec vin[],rvec vout[])
