@@ -30,6 +30,10 @@ distribution.
 #   include <cstddef>
 #endif
 
+#if defined(__GNUC__) && __GNUC__>=7
+#pragma GCC diagnostic ignored "-Wimplicit-fallthrough"
+#endif
+
 static const char LINE_FEED				= (char)0x0a;			// all line endings are normalized to LF
 static const char LF = LINE_FEED;
 static const char CARRIAGE_RETURN		= (char)0x0d;			// CR gets filtered out
