@@ -1,7 +1,7 @@
 /*
  * This file is part of the GROMACS molecular simulation package.
  *
- * Copyright (c) 2012,2013,2014, by the GROMACS development team, led by
+ * Copyright (c) 2012,2013,2014,2017, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -173,7 +173,7 @@ class AnalysisDataValue
 };
 
 //! Shorthand for reference to an array of data values.
-typedef ConstArrayRef<AnalysisDataValue> AnalysisDataValuesRef;
+typedef ArrayRef<const AnalysisDataValue> AnalysisDataValuesRef;
 
 
 /*! \brief
@@ -323,7 +323,7 @@ class AnalysisDataPointSetInfo
 };
 
 //! Shorthand for reference to an array of point set data objects.
-typedef ConstArrayRef<AnalysisDataPointSetInfo> AnalysisDataPointSetInfosRef;
+typedef ArrayRef<const AnalysisDataPointSetInfo> AnalysisDataPointSetInfosRef;
 
 //! \endcond
 
