@@ -1,7 +1,7 @@
 /*
  * This file is part of the GROMACS molecular simulation package.
  *
- * Copyright (c) 2010,2011,2012,2013,2014, by the GROMACS development team, led by
+ * Copyright (c) 2010,2011,2012,2013,2014,2015, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -540,7 +540,7 @@ class BasicHistogramImpl
 {
     public:
         //! Smart pointer to manage an BasicAverageHistogramModule object.
-        typedef boost::shared_ptr<BasicAverageHistogramModule>
+        typedef std::shared_ptr<BasicAverageHistogramModule>
             BasicAverageHistogramModulePointer;
 
         BasicHistogramImpl();
