@@ -1,7 +1,7 @@
 /*
  * This file is part of the GROMACS molecular simulation package.
  *
- * Copyright (c) 2013,2014, by the GROMACS development team, led by
+ * Copyright (c) 2013,2014,2015, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -107,7 +107,7 @@ class AnalysisDataLifetimeModule : public AbstractAnalysisArrayData,
 };
 
 //! Smart pointer to manage an AnalysisDataLifetimeModule object.
-typedef boost::shared_ptr<AnalysisDataLifetimeModule>
+typedef std::shared_ptr<AnalysisDataLifetimeModule>
     AnalysisDataLifetimeModulePointer;
 
 } // namespace gmx
