@@ -1,7 +1,7 @@
 /*
  * This file is part of the GROMACS molecular simulation package.
  *
- * Copyright (c) 2011,2012,2013,2014, by the GROMACS development team, led by
+ * Copyright (c) 2011,2012,2013,2014,2015, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -34,7 +34,7 @@
  */
 /*! \libinternal \file
  * \brief
- * Declares mock implementation of gmx::AnalysisDataModuleInterface.
+ * Declares mock implementation of gmx::IAnalysisDataModule.
  *
  * Requires Google Mock.
  *
@@ -45,7 +45,8 @@
 #ifndef GMX_ANALYSISDATA_TESTS_MOCK_DATAMODULE_H
 #define GMX_ANALYSISDATA_TESTS_MOCK_DATAMODULE_H
 
-#include <boost/shared_ptr.hpp>
+#include <memory>
+
 #include <gmock/gmock.h>
 
 #include "gromacs/analysisdata/dataframe.h"
@@ -61,7 +62,7 @@ namespace test
 class AnalysisDataTestInput;
 class TestReferenceChecker;
 
-class MockAnalysisDataModule : public AnalysisDataModuleInterface
+class MockAnalysisDataModule : public IAnalysisDataModule
 {
     public:
         explicit MockAnalysisDataModule(int flags);
@@ -98,7 +99,7 @@ class MockAnalysisDataModule : public AnalysisDataModuleInterface
 };
 
 //! Smart pointer to manage an MockAnalysisDataModule object.
-typedef boost::shared_ptr<MockAnalysisDataModule>
+typedef std::shared_ptr<MockAnalysisDataModule>
     MockAnalysisDataModulePointer;
 
 } // namespace test
