@@ -1,7 +1,7 @@
 /*
  * This file is part of the GROMACS molecular simulation package.
  *
- * Copyright (c) 2010,2011,2012,2013,2014,2015, by the GROMACS development team, led by
+ * Copyright (c) 2010,2011,2012,2013,2014,2015,2016,2017, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -48,8 +48,6 @@
 #include <algorithm>
 #include <string>
 
-#include <boost/scoped_ptr.hpp>
-
 #include "gromacs/commandline/cmdlinehelpcontext.h"
 #include "gromacs/onlinehelp/helpwritercontext.h"
 #include "gromacs/options/basicoptions.h"
@@ -59,8 +57,8 @@
 #include "gromacs/options/timeunitmanager.h"
 #include "gromacs/utility/arrayref.h"
 #include "gromacs/utility/exceptions.h"
-#include "gromacs/utility/file.h"
 #include "gromacs/utility/stringutil.h"
+#include "gromacs/utility/textwriter.h"
 
 #include "shellcompletions.h"
 
@@ -74,70 +72,7 @@ namespace
 //! \{
 
 /********************************************************************
- * DescriptionsFormatter
- */
-
-class DescriptionsFormatter : public OptionsVisitor
-{
-    public:
-        /*! \brief
-         * Creates a new description formatter.
-         *
-         * \param[in] context   Help context to use to write the help.
-         */
-        explicit DescriptionsFormatter(const HelpWriterContext &context)
-            : context_(context), title_(NULL), bDidOutput_(false)
-        {
-        }
-
-        //! Formats all section descriptions from a given options.
-        void format(const Options &options, const char *title)
-        {
-            title_      = title;
-            bDidOutput_ = false;
-            visitSubSection(options);
-            if (bDidOutput_)
-            {
-                context_.outputFile().writeLine();
-            }
-        }
-
-        //! Formats the description for a single subsection and handles recursion.
-        virtual void visitSubSection(const Options &section);
-        // This method is not used and never called.
-        virtual void visitOption(const OptionInfo & /*option*/) {}
-
-    private:
-        const HelpWriterContext &context_;
-        const char              *title_;
-        bool                     bDidOutput_;
-
-        GMX_DISALLOW_COPY_AND_ASSIGN(DescriptionsFormatter);
-};
-
-void DescriptionsFormatter::visitSubSection(const Options &section)
-{
-    if (!section.description().empty())
-    {
-        if (bDidOutput_)
-        {
-            context_.outputFile().writeLine();
-        }
-        else if (title_ != NULL)
-        {
-            context_.writeTitle(title_);
-        }
-        // TODO: Print title for the section?
-        context_.writeTextBlock(section.description());
-        bDidOutput_ = true;
-    }
-
-    OptionsIterator iterator(section);
-    iterator.acceptSubSections(this);
-}
-
-/********************************************************************
- * OptionsFormatterInterface
+ * IOptionsFormatter
  */
 
 /*! \brief
@@ -145,10 +80,10 @@ void DescriptionsFormatter::visitSubSection(const Options &section)
  *
  * \see OptionsFilter
  */
-class OptionsFormatterInterface
+class IOptionsFormatter
 {
     public:
-        virtual ~OptionsFormatterInterface() {}
+        virtual ~IOptionsFormatter() {}
 
         //! Formats a single option option.
         virtual void formatOption(const OptionInfo &option) = 0;
@@ -163,7 +98,7 @@ class OptionsFormatterInterface
  *
  * Together with code in CommandLineHelpWriter::writeHelp(), this class
  * implements the common logic for writing out the help.
- * An object implementing the OptionsFormatterInterface must be provided to the
+ * An object implementing the IOptionsFormatter must be provided to the
  * constructor, and does the actual formatting that is specific to the output
  * format.
  */
@@ -185,7 +120,7 @@ class OptionsFilter : public OptionsVisitor
          * Does not throw.
          */
         OptionsFilter()
-            : formatter_(NULL), filterType_(eSelectOtherOptions),
+            : formatter_(nullptr), filterType_(eSelectOtherOptions),
               bShowHidden_(false)
         {
         }
@@ -198,14 +133,14 @@ class OptionsFilter : public OptionsVisitor
 
         //! Formats selected options using the formatter.
         void formatSelected(FilterType                 type,
-                            OptionsFormatterInterface *formatter,
+                            IOptionsFormatter         *formatter,
                             const Options             &options);
 
-        virtual void visitSubSection(const Options &section);
+        virtual void visitSection(const OptionSectionInfo &section);
         virtual void visitOption(const OptionInfo &option);
 
     private:
-        OptionsFormatterInterface      *formatter_;
+        IOptionsFormatter              *formatter_;
         FilterType                      filterType_;
         bool                            bShowHidden_;
 
@@ -213,18 +148,18 @@ class OptionsFilter : public OptionsVisitor
 };
 
 void OptionsFilter::formatSelected(FilterType                 type,
-                                   OptionsFormatterInterface *formatter,
+                                   IOptionsFormatter         *formatter,
                                    const Options             &options)
 {
     formatter_  = formatter;
     filterType_ = type;
-    visitSubSection(options);
+    visitSection(options.rootSection());
 }
 
-void OptionsFilter::visitSubSection(const Options &section)
+void OptionsFilter::visitSection(const OptionSectionInfo &section)
 {
     OptionsIterator iterator(section);
-    iterator.acceptSubSections(this);
+    iterator.acceptSections(this);
     iterator.acceptOptions(this);
 }
 
@@ -235,7 +170,7 @@ void OptionsFilter::visitOption(const OptionInfo &option)
         return;
     }
     const FileNameOptionInfo *const fileOption = option.toType<FileNameOptionInfo>();
-    if (fileOption != NULL && fileOption->isInputFile())
+    if (fileOption != nullptr && fileOption->isInputFile())
     {
         if (filterType_ == eSelectInputFileOptions)
         {
@@ -243,7 +178,7 @@ void OptionsFilter::visitOption(const OptionInfo &option)
         }
         return;
     }
-    if (fileOption != NULL && fileOption->isInputOutputFile())
+    if (fileOption != nullptr && fileOption->isInputOutputFile())
     {
         if (filterType_ == eSelectInputOutputFileOptions)
         {
@@ -251,7 +186,7 @@ void OptionsFilter::visitOption(const OptionInfo &option)
         }
         return;
     }
-    if (fileOption != NULL && fileOption->isOutputFile())
+    if (fileOption != nullptr && fileOption->isOutputFile())
     {
         if (filterType_ == eSelectOutputFileOptions)
         {
@@ -313,27 +248,9 @@ void formatOptionNameAndValue(const OptionInfo &option, std::string *name,
 }
 
 //! Formats the default option value as a string.
-std::string
-defaultOptionValue(const OptionInfo &option)
+std::string defaultOptionValue(const OptionInfo &option)
 {
-    if (option.valueCount() == 0
-        || (option.valueCount() == 1 && option.formatValue(0).empty()))
-    {
-        return option.formatDefaultValueIfSet();
-    }
-    else
-    {
-        std::string result;
-        for (int i = 0; i < option.valueCount(); ++i)
-        {
-            if (i != 0)
-            {
-                result.append(" ");
-            }
-            result.append(option.formatValue(i));
-        }
-        return result;
-    }
+    return joinStrings(option.defaultValuesAsStrings(), " ");
 }
 
 //! Formats the flags for a file option as a string.
@@ -365,8 +282,8 @@ descriptionWithOptionDetails(const CommonFormatterData &common,
 
     const FloatOptionInfo  *floatOption  = option.toType<FloatOptionInfo>();
     const DoubleOptionInfo *doubleOption = option.toType<DoubleOptionInfo>();
-    if ((floatOption != NULL && floatOption->isTime())
-        || (doubleOption != NULL && doubleOption->isTime()))
+    if ((floatOption != nullptr && floatOption->isTime())
+        || (doubleOption != nullptr && doubleOption->isTime()))
     {
         // TODO: It could be nicer to have this in basicoptions.cpp.
         description = replaceAll(description, "%t", common.timeUnit);
@@ -382,7 +299,7 @@ descriptionWithOptionDetails(const CommonFormatterData &common,
 /*! \brief
  * Formatter implementation for synopsis.
  */
-class SynopsisFormatter : public OptionsFormatterInterface
+class SynopsisFormatter : public IOptionsFormatter
 {
     public:
         //! Creates a helper object for formatting the synopsis.
@@ -413,7 +330,7 @@ void SynopsisFormatter::start(const char *name)
 {
     currentLength_ = std::strlen(name) + 1;
     indent_        = std::min(currentLength_, 13);
-    File &file = context_.outputFile();
+    TextWriter &file = context_.outputFile();
     switch (context_.outputFormat())
     {
         case eHelpOutputFormat_Console:
@@ -436,9 +353,7 @@ void SynopsisFormatter::start(const char *name)
 
 void SynopsisFormatter::finish()
 {
-    File &file = context_.outputFile();
-    file.writeLine();
-    file.writeLine();
+    context_.outputFile().ensureLineBreak();
 }
 
 void SynopsisFormatter::formatOption(const OptionInfo &option)
@@ -456,7 +371,7 @@ void SynopsisFormatter::formatOption(const OptionInfo &option)
     }
     fullOptionText.append(bFormatted_ ? "`]" : "]");
 
-    File       &file = context_.outputFile();
+    TextWriter &file = context_.outputFile();
     currentLength_ += totalLength;
     if (currentLength_ >= lineLength_)
     {
@@ -473,7 +388,7 @@ void SynopsisFormatter::formatOption(const OptionInfo &option)
 /*! \brief
  * Formatter implementation for help export.
  */
-class OptionsListFormatter : public OptionsFormatterInterface
+class OptionsListFormatter : public IOptionsFormatter
 {
     public:
         //! Creates a helper object for formatting options.
@@ -493,7 +408,6 @@ class OptionsListFormatter : public OptionsFormatterInterface
             if (bDidOutput_)
             {
                 context_.writeOptionListEnd();
-                context_.outputFile().writeLine();
             }
         }
 
@@ -502,17 +416,18 @@ class OptionsListFormatter : public OptionsFormatterInterface
     private:
         void writeSectionStartIfNecessary()
         {
-            if (title_ != NULL)
+            if (title_ != nullptr)
             {
                 context_.writeTitle(title_);
-                title_ = NULL;
+                title_ = nullptr;
             }
             if (!bDidOutput_)
             {
-                if (header_ != NULL)
+                if (header_ != nullptr)
                 {
+                    context_.paragraphBreak();
                     context_.writeTextBlock(header_);
-                    context_.writeTextBlock("");
+                    context_.paragraphBreak();
                 }
                 context_.writeOptionListStart();
             }
@@ -533,7 +448,7 @@ OptionsListFormatter::OptionsListFormatter(
         const CommonFormatterData &common,
         const char                *title)
     : context_(context), common_(common),
-      title_(title), header_(NULL), bDidOutput_(false)
+      title_(title), header_(nullptr), bDidOutput_(false)
 {
 }
 
@@ -545,7 +460,7 @@ void OptionsListFormatter::formatOption(const OptionInfo &option)
     std::string               defaultValue(defaultOptionValue(option));
     std::string               info;
     const FileNameOptionInfo *fileOption = option.toType<FileNameOptionInfo>();
-    if (fileOption != NULL)
+    if (fileOption != nullptr)
     {
         const bool bAbbrev = (context_.outputFormat() == eHelpOutputFormat_Console);
         info = fileOptionFlagsAsString(*fileOption, bAbbrev);
@@ -571,27 +486,22 @@ class CommandLineHelpWriter::Impl
 {
     public:
         //! Sets the Options object to use for generating help.
-        explicit Impl(const Options &options);
+        explicit Impl(const Options &options)
+            : options_(options)
+        {
+        }
 
         //! Format the list of known issues.
         void formatBugs(const HelpWriterContext &context);
 
         //! Options object to use for generating help.
         const Options               &options_;
+        //! Help text.
+        std::string                  helpText_;
         //! List of bugs/knows issues.
-        ConstArrayRef<const char *>  bugs_;
-        //! Time unit to show in descriptions.
-        std::string                  timeUnit_;
-        //! Whether to write descriptions to output.
-        bool                         bShowDescriptions_;
+        ArrayRef<const char *const>  bugs_;
 };
 
-CommandLineHelpWriter::Impl::Impl(const Options &options)
-    : options_(options), timeUnit_(TimeUnitManager().timeUnitAsString()),
-      bShowDescriptions_(false)
-{
-}
-
 void CommandLineHelpWriter::Impl::formatBugs(const HelpWriterContext &context)
 {
     if (bugs_.empty())
@@ -599,7 +509,7 @@ void CommandLineHelpWriter::Impl::formatBugs(const HelpWriterContext &context)
         return;
     }
     context.writeTitle("Known Issues");
-    ConstArrayRef<const char *>::const_iterator i;
+    ArrayRef<const char *const>::const_iterator i;
     for (i = bugs_.begin(); i != bugs_.end(); ++i)
     {
         const char *const       bug = *i;
@@ -622,21 +532,21 @@ CommandLineHelpWriter::~CommandLineHelpWriter()
 }
 
 CommandLineHelpWriter &
-CommandLineHelpWriter::setShowDescriptions(bool bSet)
+CommandLineHelpWriter::setHelpText(const std::string &help)
 {
-    impl_->bShowDescriptions_ = bSet;
+    impl_->helpText_ = help;
     return *this;
 }
 
 CommandLineHelpWriter &
-CommandLineHelpWriter::setTimeUnitString(const char *timeUnit)
+CommandLineHelpWriter::setHelpText(const ArrayRef<const char *const> &help)
 {
-    impl_->timeUnit_ = timeUnit;
+    impl_->helpText_ = joinStrings(help, "\n");
     return *this;
 }
 
 CommandLineHelpWriter &
-CommandLineHelpWriter::setKnownIssues(const ConstArrayRef<const char *> &bugs)
+CommandLineHelpWriter::setKnownIssues(const ArrayRef<const char *const> &bugs)
 {
     impl_->bugs_ = bugs;
     return *this;
@@ -669,12 +579,12 @@ void CommandLineHelpWriter::writeHelp(const CommandLineHelpContext &context)
         synopsisFormatter.finish();
     }
 
-    if (impl_->bShowDescriptions_)
+    if (!impl_->helpText_.empty())
     {
-        DescriptionsFormatter descriptionFormatter(writerContext);
-        descriptionFormatter.format(impl_->options_, "Description");
+        writerContext.writeTitle("Description");
+        writerContext.writeTextBlock(impl_->helpText_);
     }
-    CommonFormatterData    common(impl_->timeUnit_.c_str());
+    CommonFormatterData    common(TimeUnitManager().timeUnitAsString());
     OptionsListFormatter   formatter(writerContext, common, "Options");
     formatter.startSection("Options to specify input files:");
     filter.formatSelected(OptionsFilter::eSelectInputFileOptions,
