@@ -1,7 +1,7 @@
 /*
  * This file is part of the GROMACS molecular simulation package.
  *
- * Copyright (c) 2013,2014,2015, by the GROMACS development team, led by
+ * Copyright (c) 2013,2014,2015,2017, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -43,17 +43,14 @@
 
 #include "cmdlineinit.h"
 
-#include "config.h"
-
 #include <cstring>
 
-#include <boost/scoped_ptr.hpp>
+#include <memory>
 
 #include "gromacs/commandline/cmdlinemodulemanager.h"
 #include "gromacs/commandline/cmdlineoptionsmodule.h"
 #include "gromacs/commandline/cmdlineprogramcontext.h"
-#include "gromacs/legacyheaders/network.h"
-#include "gromacs/legacyheaders/types/commrec.h"
+#include "gromacs/utility/basenetwork.h"
 #include "gromacs/utility/datafilefinder.h"
 #include "gromacs/utility/exceptions.h"
 #include "gromacs/utility/futil.h"
@@ -71,36 +68,47 @@ namespace
 //! \addtogroup module_commandline
 //! \{
 
+// These never release ownership.
 //! Global context instance initialized in initForCommandLine().
-boost::scoped_ptr<CommandLineProgramContext> g_commandLineContext;
+std::unique_ptr<CommandLineProgramContext> g_commandLineContext;
 //! Global library data file finder that respects GMXLIB.
-boost::scoped_ptr<DataFileFinder>            g_libFileFinder;
+std::unique_ptr<DataFileFinder>            g_libFileFinder;
 
-#ifdef GMX_LIB_MPI
-void broadcastArguments(const t_commrec *cr, int *argc, char ***argv)
+/*! \brief
+ * Broadcasts command-line arguments to all ranks.
+ *
+ * MPI does not ensure that command-line arguments would be passed on any
+ * other rank than zero, but our code wants to parse them on each rank
+ * separately.
+ */
+void broadcastArguments(int *argc, char ***argv)
 {
-    gmx_bcast(sizeof(*argc), argc, cr);
+    if (gmx_node_num() <= 1)
+    {
+        return;
+    }
+    gmx_broadcast_world(sizeof(*argc), argc);
 
-    if (!MASTER(cr))
+    const bool isMaster = (gmx_node_rank() == 0);
+    if (!isMaster)
     {
         snew(*argv, *argc+1);
     }
     for (int i = 0; i < *argc; i++)
     {
         int len;
-        if (MASTER(cr))
+        if (isMaster)
         {
             len = std::strlen((*argv)[i])+1;
         }
-        gmx_bcast(sizeof(len), &len, cr);
-        if (!MASTER(cr))
+        gmx_broadcast_world(sizeof(len), &len);
+        if (!isMaster)
         {
             snew((*argv)[i], len);
         }
-        gmx_bcast(len, (*argv)[i], cr);
+        gmx_broadcast_world(len, (*argv)[i]);
     }
 }
-#endif
 
 //! \}
 
@@ -111,20 +119,9 @@ CommandLineProgramContext &initForCommandLine(int *argc, char ***argv)
     gmx::init(argc, argv);
     GMX_RELEASE_ASSERT(!g_commandLineContext,
                        "initForCommandLine() calls cannot be nested");
-#ifdef GMX_LIB_MPI
-    // TODO: Rewrite this to not use t_commrec once there is clarity on
-    // the approach for MPI in C++ code.
     // TODO: Consider whether the argument broadcast would better be done
     // in CommandLineModuleManager.
-    t_commrec cr;
-    std::memset(&cr, 0, sizeof(cr));
-
-    gmx_fill_commrec_from_mpi(&cr);
-    if (PAR(&cr))
-    {
-        broadcastArguments(&cr, argc, argv);
-    }
-#endif
+    broadcastArguments(argc, argv);
     try
     {
         g_commandLineContext.reset(new CommandLineProgramContext(*argc, *argv));
@@ -144,30 +141,30 @@ CommandLineProgramContext &initForCommandLine(int *argc, char ***argv)
 void finalizeForCommandLine()
 {
     gmx::finalize();
-    setLibraryFileFinder(NULL);
+    setLibraryFileFinder(nullptr);
     g_libFileFinder.reset();
-    setProgramContext(NULL);
+    setProgramContext(nullptr);
     g_commandLineContext.reset();
 }
 
 int processExceptionAtExitForCommandLine(const std::exception &ex)
 {
-    int rc = processExceptionAtExit(ex); //Currently this aborts for GMX_LIB_MPI
-    finalizeForCommandLine();            //thus this MPI_Finalize doesn't matter
+    int rc = processExceptionAtExit(ex); // Currently this aborts for real MPI
+    finalizeForCommandLine();            // thus this MPI_Finalize doesn't matter.
     return rc;
 }
 
 int runCommandLineModule(int argc, char *argv[],
-                         CommandLineModuleInterface *module)
+                         ICommandLineModule *module)
 {
     return CommandLineModuleManager::runAsMainSingleModule(argc, argv, module);
 }
 
-int runCommandLineModule(int argc, char *argv[],
-                         const char *name, const char *description,
-                         CommandLineOptionsModuleInterface *(*factory)())
+int runCommandLineModule(
+        int argc, char *argv[], const char *name, const char *description,
+        std::function<std::unique_ptr<ICommandLineOptionsModule>()> factory)
 {
-    return CommandLineOptionsModuleInterface::runAsMain(
+    return ICommandLineOptionsModule::runAsMain(
             argc, argv, name, description, factory);
 }
 
