@@ -1,7 +1,7 @@
 /*
  * This file is part of the GROMACS molecular simulation package.
  *
- * Copyright (c) 2012,2013,2014,2015, by the GROMACS development team, led by
+ * Copyright (c) 2012,2013,2014,2015,2016, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -43,24 +43,26 @@
 #ifndef GMX_COMMANDLINE_CMDLINEMODULEMANAGER_H
 #define GMX_COMMANDLINE_CMDLINEMODULEMANAGER_H
 
-#include "gromacs/onlinehelp/helptopicinterface.h"
+#include <memory>
+
+#include "gromacs/onlinehelp/ihelptopic.h"
 #include "gromacs/utility/classhelpers.h"
-#include "gromacs/utility/uniqueptr.h"
 
 namespace gmx
 {
 
 class CommandLineModuleGroup;
 class CommandLineModuleGroupData;
-class CommandLineModuleInterface;
+class CommandLineModuleSettings;
 class CommandLineProgramContext;
-class FileOutputRedirectorInterface;
+class ICommandLineModule;
+class IFileOutputRedirector;
 
 //! \addtogroup module_commandline
 //! \{
 
-//! Smart pointer type for managing a CommandLineModuleInterface.
-typedef gmx_unique_ptr<CommandLineModuleInterface>::type
+//! Smart pointer type for managing a ICommandLineModule.
+typedef std::unique_ptr<ICommandLineModule>
     CommandLineModulePointer;
 
 /*! \libinternal \brief
@@ -95,6 +97,8 @@ class CommandLineModuleManager
     public:
         //! Function pointer type for a C main function.
         typedef int (*CMainFunction)(int argc, char *argv[]);
+        //! Function pointer to a settings provider.
+        typedef void (*InitSettingsFunction)(CommandLineModuleSettings *settings);
 
         /*! \brief
          * Implements a main() method that runs a single module.
@@ -127,7 +131,7 @@ class CommandLineModuleManager
          * Does not throw.  All exceptions are caught and handled internally.
          */
         static int runAsMainSingleModule(int argc, char *argv[],
-                                         CommandLineModuleInterface *module);
+                                         ICommandLineModule *module);
         /*! \brief
          * Implements a main() method that runs a given function.
          *
@@ -159,6 +163,17 @@ class CommandLineModuleManager
          */
         static int runAsMainCMain(int argc, char *argv[],
                                   CMainFunction mainFunction);
+        /*! \brief
+         * Implements a main() method that runs a given function with custom
+         * settings.
+         *
+         * This method does the same as runAsMainCMain(), but additionally
+         * calls \p settingsFunction to initialize CommandLineModuleSettings.
+         * This allows specifying, e.g., a different default nice level.
+         */
+        static int runAsMainCMainWithSettings(int argc, char *argv[],
+                                              CMainFunction mainFunction,
+                                              InitSettingsFunction settingsFunction);
 
         /*! \brief
          * Initializes a command-line module manager.
@@ -206,7 +221,7 @@ class CommandLineModuleManager
          * For tests, there should only be need to call this a single time,
          * right after creating the manager.
          */
-        void setOutputRedirector(FileOutputRedirectorInterface *output);
+        void setOutputRedirector(IFileOutputRedirector *output);
 
         /*! \brief
          * Makes the manager always run a single module.
@@ -217,7 +232,7 @@ class CommandLineModuleManager
          * directly passes all command-line arguments to \p module.
          * Help arguments are an exception: these are still recognized by the
          * manager and translated into a call to
-         * CommandLineModuleInterface::writeHelp().
+         * ICommandLineModule::writeHelp().
          *
          * This is public mainly for unit testing purposes; for other code,
          * runAsMainSingleModule() typically provides the desired
@@ -225,7 +240,7 @@ class CommandLineModuleManager
          *
          * Does not throw.
          */
-        void setSingleModule(CommandLineModuleInterface *module);
+        void setSingleModule(ICommandLineModule *module);
         /*! \brief
          * Adds a given module to this manager.
          *
@@ -259,6 +274,17 @@ class CommandLineModuleManager
          */
         void addModuleCMain(const char *name, const char *shortDescription,
                             CMainFunction mainFunction);
+        /*! \brief
+         * Adds a module that runs a given main()-like function with custom
+         * settings.
+         *
+         * This method does the same as runAsMainCMain(), but additionally
+         * calls \p settingsFunction to initialize CommandLineModuleSettings.
+         * This allows specifying, e.g., a different default nice level.
+         */
+        void addModuleCMainWithSettings(const char *name, const char *shortDescription,
+                                        CMainFunction mainFunction,
+                                        InitSettingsFunction settingsFunction);
         /*! \brief
          * Registers a module of a certain type to this manager.
          *
@@ -266,7 +292,7 @@ class CommandLineModuleManager
          * \throws  std::bad_alloc if out of memory.
          *
          * \p Module must be default-constructible and implement
-         * CommandLineModuleInterface.
+         * ICommandLineModule.
          *
          * This method is provided as a convenient alternative to addModule()
          * for cases where each module is implemented by a different type
