@@ -1,7 +1,7 @@
 /*
  * This file is part of the GROMACS molecular simulation package.
  *
- * Copyright (c) 2014,2015, by the GROMACS development team, led by
+ * Copyright (c) 2014,2015,2017, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -51,7 +51,7 @@
 namespace gmx
 {
 
-template <typename T> class ConstArrayRef;
+template <typename T> class ArrayRef;
 
 class CommandLineModuleManager;
 class ICommandLineModule;
@@ -91,7 +91,7 @@ class ICommandLineOptionsModuleSettings
            settings->setHelpText(desc);
            \endcode
          */
-        virtual void setHelpText(const ConstArrayRef<const char *> &help) = 0;
+        virtual void setHelpText(const ArrayRef<const char *const> &help) = 0;
         /*! \brief
          * Adds an option behavior that performs actions before
          * ICommandLineOptionsModule::run() is called.
