@@ -3,7 +3,7 @@
  *
  * Copyright (c) 1991-2000, University of Groningen, The Netherlands.
  * Copyright (c) 2001-2004, The GROMACS development team.
- * Copyright (c) 2013,2014,2015,2016, by the GROMACS development team, led by
+ * Copyright (c) 2013,2014,2015,2016,2017, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -62,7 +62,7 @@ const t_filenm *getFilenm(const char *opt, int nfile, const t_filenm fnm[])
         }
     }
 
-    return NULL;
+    return nullptr;
 }
 
 const char *opt2fn(const char *opt, int nfile, const t_filenm fnm[])
@@ -79,7 +79,7 @@ const char *opt2fn(const char *opt, int nfile, const t_filenm fnm[])
 
     fprintf(stderr, "No option %s\n", opt);
 
-    return NULL;
+    return nullptr;
 }
 
 int opt2fns(char **fns[], const char *opt, int nfile, const t_filenm fnm[])
@@ -112,7 +112,7 @@ const char *ftp2fn(int ftp, int nfile, const t_filenm fnm[])
     }
 
     fprintf(stderr, "ftp2fn: No filetype %s\n", ftp2ext_with_dot(ftp));
-    return NULL;
+    return nullptr;
 }
 
 int ftp2fns(char **fns[], int ftp, int nfile, const t_filenm fnm[])
@@ -176,7 +176,7 @@ const char *opt2fn_null(const char *opt, int nfile, const t_filenm fnm[])
         {
             if (IS_OPT(fnm[i]) && !IS_SET(fnm[i]))
             {
-                return NULL;
+                return nullptr;
             }
             else
             {
@@ -185,7 +185,7 @@ const char *opt2fn_null(const char *opt, int nfile, const t_filenm fnm[])
         }
     }
     fprintf(stderr, "No option %s\n", opt);
-    return NULL;
+    return nullptr;
 }
 
 const char *ftp2fn_null(int ftp, int nfile, const t_filenm fnm[])
@@ -198,7 +198,7 @@ const char *ftp2fn_null(int ftp, int nfile, const t_filenm fnm[])
         {
             if (IS_OPT(fnm[i]) && !IS_SET(fnm[i]))
             {
-                return NULL;
+                return nullptr;
             }
             else
             {
@@ -207,7 +207,7 @@ const char *ftp2fn_null(int ftp, int nfile, const t_filenm fnm[])
         }
     }
     fprintf(stderr, "ftp2fn: No filetype %s\n", ftp2ext_with_dot(ftp));
-    return NULL;
+    return nullptr;
 }
 
 gmx_bool is_optional(const t_filenm *fnm)
@@ -266,7 +266,7 @@ t_filenm *dup_tfn(int nf, const t_filenm tfn[])
         }
         else
         {
-            ret[i].opt = NULL;
+            ret[i].opt = nullptr;
         }
 
         if (tfn[i].fn)
@@ -275,7 +275,7 @@ t_filenm *dup_tfn(int nf, const t_filenm tfn[])
         }
         else
         {
-            ret[i].fn = NULL;
+            ret[i].fn = nullptr;
         }
 
         if (tfn[i].nfiles > 0)
@@ -301,6 +301,6 @@ void done_filenms(int nf, t_filenm fnm[])
             sfree(fnm[i].fns[j]);
         }
         sfree(fnm[i].fns);
-        fnm[i].fns = NULL;
+        fnm[i].fns = nullptr;
     }
 }
