@@ -1,7 +1,7 @@
 /*
  * This file is part of the GROMACS molecular simulation package.
  *
- * Copyright (c) 2005,2006,2007,2008,2009,2010,2012,2013,2014,2015, by the GROMACS development team, led by
+ * Copyright (c) 2005,2006,2007,2008,2009,2010,2012,2013,2014,2015,2016,2017, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -60,30 +60,33 @@
 
 #include <stdio.h>
 
+#include <vector>
+
 #include "gromacs/gmxlib/nrnb.h"
-#include "gromacs/hardware/hw_info.h"
 #include "gromacs/math/vectypes.h"
-#include "gromacs/mdlib/constr.h"
 #include "gromacs/mdlib/vsite.h"
 #include "gromacs/mdtypes/forcerec.h"
 #include "gromacs/mdtypes/mdatom.h"
-#include "gromacs/mdtypes/state.h"
 #include "gromacs/timing/wallcycle.h"
 #include "gromacs/topology/block.h"
 #include "gromacs/topology/idef.h"
 #include "gromacs/topology/topology.h"
+#include "gromacs/utility/arrayref.h"
 #include "gromacs/utility/basedefinitions.h"
 #include "gromacs/utility/real.h"
 
 struct gmx_domdec_t;
 struct gmx_ddbox_t;
 struct gmx_domdec_zones_t;
+struct MdrunOptions;
 struct t_commrec;
 struct t_inputrec;
+class t_state;
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+namespace gmx
+{
+class MDAtoms;
+} // namespace
 
 /*! \brief Returns the global topology atom number belonging to local atom index i.
  *
@@ -109,6 +112,9 @@ struct gmx_domdec_zones_t *domdec_zones(struct gmx_domdec_t *dd);
 void dd_get_ns_ranges(const gmx_domdec_t *dd, int icg,
                       int *jcg0, int *jcg1, ivec shift0, ivec shift1);
 
+/*! \brief Returns the atom range in the local state for atoms that need to be present in mdatoms */
+int dd_natoms_mdatoms(const gmx_domdec_t *dd);
+
 /*! \brief Returns the atom range in the local state for atoms involved in virtual sites */
 int dd_natoms_vsite(const gmx_domdec_t *dd);
 
@@ -120,9 +126,8 @@ void dd_get_constraint_range(const gmx_domdec_t *dd,
 void get_pme_nnodes(const struct gmx_domdec_t *dd,
                     int *npmenodes_x, int *npmenodes_y);
 
-/*! \brief Returns the set of DD nodes that communicate with pme node cr->nodeid */
-void get_pme_ddnodes(struct t_commrec *cr, int pmenodeid,
-                     int *nmy_ddnodes, int **my_ddnodes, int *node_peer);
+/*! \brief Returns the set of DD ranks that communicate with pme node cr->nodeid */
+std::vector<int> get_pme_ddranks(t_commrec *cr, int pmenodeid);
 
 /*! \brief Returns the maximum shift for coordinate communication in PME, dim x */
 int dd_pme_maxshift_x(const gmx_domdec_t *dd);
@@ -130,27 +135,73 @@ int dd_pme_maxshift_x(const gmx_domdec_t *dd);
 /*! \brief Returns the maximum shift for coordinate communication in PME, dim y */
 int dd_pme_maxshift_y(const gmx_domdec_t *dd);
 
+/*! \brief The options for the domain decomposition MPI task ordering. */
+enum class DdRankOrder
+{
+    select,     //!< First value (needed to cope with command-line parsing)
+    interleave, //!< Interleave the PP and PME ranks
+    pp_pme,     //!< First all PP ranks, all PME rank at the end
+    cartesian,  //!< Use Cartesian communicators for PP, PME and PP-PME
+    nr          //!< The number of options
+};
+
+/*! \brief The options for the dynamic load balancing. */
+enum class DlbOption
+{
+    select,           //!< First value (needed to cope with command-line parsing)
+    turnOnWhenUseful, //!< Turn on DLB when we think it would improve performance
+    no,               //!< Never turn on DLB
+    yes,              //!< Turn on DLB from the start and keep it on
+    nr                //!< The number of options
+};
+
+/*! \libinternal \brief Structure containing all (command line) options for the domain decomposition */
+struct DomdecOptions
+{
+    /*! \brief Constructor */
+    DomdecOptions();
+
+    //! If true, check that all bonded interactions have been assigned to exactly one domain/rank.
+    gmx_bool          checkBondedInteractions;
+    //! If true, don't communicate all atoms between the non-bonded cut-off and the larger bonded cut-off, but only those that have non-local bonded interactions. This significantly reduces the communication volume.
+    gmx_bool          useBondedCommunication;
+    //! The domain decomposition grid cell count, 0 means let domdec choose based on the number of ranks.
+    ivec              numCells;
+    //! The number of separate PME ranks requested, -1 = auto.
+    int               numPmeRanks;
+    //! Ordering of the PP and PME ranks, values from enum above.
+    DdRankOrder       rankOrder;
+    //! The minimum communication range, used for extended the communication range for bonded interactions (nm).
+    real              minimumCommunicationRange;
+    //! Communication range for atom involved in constraints (P-LINCS) (nm).
+    real              constraintCommunicationRange;
+    //! Dynamic load balancing option, values from enum above.
+    DlbOption         dlbOption;
+    /*! \brief Fraction in (0,1) by whose reciprocal the initial
+     * DD cell size will be increased in order to provide a margin
+     * in which dynamic load balancing can act, while preserving
+     * the minimum cell size. */
+    real              dlbScaling;
+    //! String containing a vector of the relative sizes in the x direction of the corresponding DD cells.
+    const char       *cellSizeX;
+    //! String containing a vector of the relative sizes in the y direction of the corresponding DD cells.
+    const char       *cellSizeY;
+    //! String containing a vector of the relative sizes in the z direction of the corresponding DD cells.
+    const char       *cellSizeZ;
+};
+
 /*! \brief Initialized the domain decomposition, chooses the DD grid and PME ranks, return the DD struct */
-gmx_domdec_t *init_domain_decomposition(FILE             *fplog,
-                                        t_commrec        *cr,
-                                        unsigned long     Flags,
-                                        ivec              nc,
-                                        int               nPmeRanks,
-                                        int               dd_node_order,
-                                        real              comm_distance_min,
-                                        real              rconstr,
-                                        const char       *dlb_opt,
-                                        real              dlb_scale,
-                                        const char       *sizex,
-                                        const char       *sizey,
-                                        const char       *sizez,
-                                        const gmx_mtop_t *mtop,
-                                        const t_inputrec *ir,
-                                        matrix            box,
-                                        rvec             *x,
-                                        gmx_ddbox_t      *ddbox,
-                                        int              *npme_x,
-                                        int              *npme_y);
+gmx_domdec_t *init_domain_decomposition(FILE                *fplog,
+                                        t_commrec           *cr,
+                                        const DomdecOptions &options,
+                                        const MdrunOptions  &mdrunOptions,
+                                        const gmx_mtop_t    *mtop,
+                                        const t_inputrec    *ir,
+                                        const matrix         box,
+                                        const rvec          *xGlobal,
+                                        gmx_ddbox_t         *ddbox,
+                                        int                 *npme_x,
+                                        int                 *npme_y);
 
 /*! \brief Initialize data structures for bonded interactions */
 void dd_init_bondeds(FILE              *fplog,
@@ -195,24 +246,25 @@ void dd_dlb_lock(struct gmx_domdec_t *dd);
 /*! \brief Clear a lock such that with DLB=auto DLB may get turned on later */
 void dd_dlb_unlock(struct gmx_domdec_t *dd);
 
-/*! \brief Set up communication for averaging GPU wait times over ranks
+/*! \brief Set up communication for averaging GPU wait times over domains
  *
  * When domains (PP MPI ranks) share a GPU, the individual GPU wait times
  * are meaningless, as it depends on the order in which tasks on the same
  * GPU finish. Therefore there wait times need to be averaged over the ranks
  * sharing the same GPU. This function sets up the communication for that.
  */
-void dd_setup_dlb_resource_sharing(struct t_commrec           *cr,
-                                   const gmx_hw_info_t        *hwinfo,
-                                   const gmx_hw_opt_t         *hw_opt);
+void dd_setup_dlb_resource_sharing(t_commrec           *cr,
+                                   int                  gpu_id);
 
 /*! \brief Collects local rvec arrays \p lv to \p v on the master rank */
-void dd_collect_vec(struct gmx_domdec_t *dd,
-                    t_state *state_local, rvec *lv, rvec *v);
+void dd_collect_vec(struct gmx_domdec_t           *dd,
+                    const t_state                 *state_local,
+                    gmx::ArrayRef<const gmx::RVec> lv,
+                    gmx::ArrayRef<gmx::RVec>       v);
 
 /*! \brief Collects the local state \p state_local to \p state on the master rank */
 void dd_collect_state(struct gmx_domdec_t *dd,
-                      t_state *state_local, t_state *state);
+                      const t_state *state_local, t_state *state);
 
 /*! \brief Cycle counter indices used internally in the domain decomposition */
 enum {
@@ -220,7 +272,7 @@ enum {
 };
 
 /*! \brief Add the wallcycle count to the DD counter */
-void dd_cycles_add(struct gmx_domdec_t *dd, float cycles, int ddCycl);
+void dd_cycles_add(const gmx_domdec_t *dd, float cycles, int ddCycl);
 
 /*! \brief Start the force flop count */
 void dd_force_flop_start(struct gmx_domdec_t *dd, t_nrnb *nrnb);
@@ -266,12 +318,12 @@ void dd_partition_system(FILE                *fplog,
                          const gmx_mtop_t    *top_global,
                          const t_inputrec    *ir,
                          t_state             *state_local,
-                         rvec               **f,
-                         t_mdatoms           *mdatoms,
+                         PaddedRVecVector    *f,
+                         gmx::MDAtoms        *mdatoms,
                          gmx_localtop_t      *top_local,
                          t_forcerec          *fr,
                          gmx_vsite_t         *vsite,
-                         gmx_constr_t         constr,
+                         struct gmx_constr   *constr,
                          t_nrnb              *nrnb,
                          gmx_wallcycle_t      wcycle,
                          gmx_bool             bVerbose);
@@ -280,7 +332,7 @@ void dd_partition_system(FILE                *fplog,
 void reset_dd_statistics_counters(struct gmx_domdec_t *dd);
 
 /*! \brief Print statistics for domain decomposition communication */
-void print_dd_statistics(struct t_commrec *cr, t_inputrec *ir, FILE *fplog);
+void print_dd_statistics(struct t_commrec *cr, const t_inputrec *ir, FILE *fplog);
 
 /* In domdec_con.c */
 
@@ -295,7 +347,7 @@ void dd_move_x_constraints(struct gmx_domdec_t *dd, matrix box,
                            rvec *x0, rvec *x1, gmx_bool bX1IsCoord);
 
 /*! \brief Communicates the coordinates involved in virtual sites */
-void dd_move_x_vsites(struct gmx_domdec_t *dd, matrix box, rvec *x);
+void dd_move_x_vsites(struct gmx_domdec_t *dd, const matrix box, rvec *x);
 
 /*! \brief Returns the local atom count array for all constraints
  *
@@ -350,7 +402,7 @@ t_blocka *make_charge_group_links(const gmx_mtop_t *mtop, gmx_domdec_t *dd,
 /*! \brief Calculate the maximum distance involved in 2-body and multi-body bonded interactions */
 void dd_bonded_cg_distance(FILE *fplog, const gmx_mtop_t *mtop,
                            const t_inputrec *ir,
-                           const rvec *x, matrix box,
+                           const rvec *x, const matrix box,
                            gmx_bool bBCheck,
                            real *r_2b, real *r_mb);
 
@@ -378,7 +430,7 @@ real dd_choose_grid(FILE *fplog,
                     t_commrec *cr, gmx_domdec_t *dd,
                     const t_inputrec *ir,
                     const gmx_mtop_t *mtop,
-                    matrix box, const gmx_ddbox_t *ddbox,
+                    const matrix box, const gmx_ddbox_t *ddbox,
                     int nPmeRanks,
                     gmx_bool bDynLoadBal, real dlb_scale,
                     real cellsize_limit, real cutoff_dd,
@@ -399,8 +451,4 @@ void set_ddbox_cr(t_commrec *cr, const ivec *dd_nc,
                   const t_block *cgs, const rvec *x,
                   gmx_ddbox_t *ddbox);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif
