@@ -1,7 +1,7 @@
 /*
  * This file is part of the GROMACS molecular simulation package.
  *
- * Copyright (c) 2006,2007,2008,2009,2010,2012,2013,2014,2015, by the GROMACS development team, led by
+ * Copyright (c) 2006,2007,2008,2009,2010,2012,2013,2014,2015,2016,2017, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -50,6 +50,7 @@
 
 #include <algorithm>
 
+#include "gromacs/domdec/dlbtiming.h"
 #include "gromacs/domdec/domdec.h"
 #include "gromacs/domdec/domdec_struct.h"
 #include "gromacs/domdec/ga2la.h"
@@ -58,13 +59,14 @@
 #include "gromacs/mdlib/gmx_omp_nthreads.h"
 #include "gromacs/mdtypes/commrec.h"
 #include "gromacs/pbcutil/ishift.h"
-#include "gromacs/topology/mtop_util.h"
+#include "gromacs/topology/mtop_lookup.h"
 #include "gromacs/utility/exceptions.h"
 #include "gromacs/utility/fatalerror.h"
 #include "gromacs/utility/gmxassert.h"
 #include "gromacs/utility/smalloc.h"
 
 #include "domdec_specatomcomm.h"
+#include "domdec_vsite.h"
 #include "hash.h"
 
 /*! \brief Struct used during constraint setup with domain decomposition */
@@ -94,6 +96,8 @@ void dd_move_x_constraints(gmx_domdec_t *dd, matrix box,
     if (dd->constraint_comm)
     {
         dd_move_x_specat(dd, dd->constraint_comm, box, x0, x1, bX1IsCoord);
+
+        ddReopenBalanceRegionCpu(dd);
     }
 }
 
@@ -105,7 +109,7 @@ int *dd_constraints_nlocalatoms(gmx_domdec_t *dd)
     }
     else
     {
-        return NULL;
+        return nullptr;
     }
 }
 
@@ -242,51 +246,37 @@ static void atoms_to_settles(gmx_domdec_t *dd,
                              t_ilist *ils_local,
                              ind_req_t *ireq)
 {
-    gmx_ga2la_t            *ga2la;
-    gmx_mtop_atomlookup_t   alook;
-    int                     settle;
-    int                     nral, sa;
-    int                     cg, a, a_gl, a_glsa, a_gls[3], a_locs[3];
-    int                     mb, molnr, a_mol, offset;
-    const gmx_molblock_t   *molb;
-    const t_iatom          *ia1;
-    gmx_bool                a_home[3];
-    int                     nlocal;
-    gmx_bool                bAssign;
+    gmx_ga2la_t *ga2la = dd->ga2la;
+    int          nral  = NRAL(F_SETTLE);
 
-    ga2la  = dd->ga2la;
-
-    alook = gmx_mtop_atomlookup_settle_init(mtop);
-
-    nral = NRAL(F_SETTLE);
-
-    for (cg = cg_start; cg < cg_end; cg++)
+    int          mb    = 0;
+    for (int cg = cg_start; cg < cg_end; cg++)
     {
         if (GET_CGINFO_SETTLE(cginfo[cg]))
         {
-            for (a = dd->cgindex[cg]; a < dd->cgindex[cg+1]; a++)
+            for (int a = dd->cgindex[cg]; a < dd->cgindex[cg+1]; a++)
             {
-                a_gl = dd->gatindex[a];
+                int a_gl = dd->gatindex[a];
+                int a_mol;
+                mtopGetMolblockIndex(mtop, a_gl, &mb, nullptr, &a_mol);
 
-                gmx_mtop_atomnr_to_molblock_ind(alook, a_gl, &mb, &molnr, &a_mol);
-                molb = &mtop->molblock[mb];
-
-                settle = at2settle_mt[molb->type][a_mol];
+                const gmx_molblock_t *molb   = &mtop->molblock[mb];
+                int                   settle = at2settle_mt[molb->type][a_mol];
 
                 if (settle >= 0)
                 {
-                    offset = a_gl - a_mol;
+                    int      offset  = a_gl - a_mol;
 
-                    ia1 = mtop->moltype[molb->type].ilist[F_SETTLE].iatoms;
+                    t_iatom *ia1     = mtop->moltype[molb->type].ilist[F_SETTLE].iatoms;
 
-                    bAssign = FALSE;
-                    nlocal  = 0;
-                    for (sa = 0; sa < nral; sa++)
+                    int      a_gls[3], a_locs[3];
+                    gmx_bool bAssign = FALSE;
+                    int      nlocal  = 0;
+                    for (int sa = 0; sa < nral; sa++)
                     {
-                        a_glsa     = offset + ia1[settle*(1+nral)+1+sa];
+                        int a_glsa = offset + ia1[settle*(1+nral)+1+sa];
                         a_gls[sa]  = a_glsa;
-                        a_home[sa] = ga2la_get_home(ga2la, a_glsa, &a_locs[sa]);
-                        if (a_home[sa])
+                        if (ga2la_get_home(ga2la, a_glsa, &a_locs[sa]))
                         {
                             if (nlocal == 0 && a_gl == a_glsa)
                             {
@@ -306,7 +296,7 @@ static void atoms_to_settles(gmx_domdec_t *dd,
 
                         ils_local->iatoms[ils_local->nr++] = ia1[settle*4];
 
-                        for (sa = 0; sa < nral; sa++)
+                        for (int sa = 0; sa < nral; sa++)
                         {
                             if (ga2la_get_home(ga2la, a_gls[sa], &a_locs[sa]))
                             {
@@ -332,8 +322,6 @@ static void atoms_to_settles(gmx_domdec_t *dd,
             }
         }
     }
-
-    gmx_mtop_atomlookup_destroy(alook);
 }
 
 /*! \brief Looks up constraint for the local atoms */
@@ -345,33 +333,28 @@ static void atoms_to_constraints(gmx_domdec_t *dd,
                                  ind_req_t *ireq)
 {
     const t_blocka             *at2con;
-    gmx_ga2la_t                *ga2la;
-    gmx_mtop_atomlookup_t       alook;
     int                         ncon1;
-    gmx_molblock_t             *molb;
     t_iatom                    *ia1, *ia2, *iap;
-    int                         nhome, cg, a, a_gl, a_mol, a_loc, b_lo, offset, mb, molnr, b_mol, i, con, con_offset;
-    gmx_domdec_constraints_t   *dc;
-    gmx_domdec_specat_comm_t   *dcc;
+    int                         a_loc, b_lo, offset, b_mol, i, con, con_offset;
 
-    dc  = dd->constraints;
-    dcc = dd->constraint_comm;
+    gmx_domdec_constraints_t   *dc     = dd->constraints;
+    gmx_domdec_specat_comm_t   *dcc    = dd->constraint_comm;
 
-    ga2la  = dd->ga2la;
+    gmx_ga2la_t                *ga2la  = dd->ga2la;
 
-    alook = gmx_mtop_atomlookup_init(mtop);
-
-    nhome = 0;
-    for (cg = 0; cg < dd->ncg_home; cg++)
+    int mb    = 0;
+    int nhome = 0;
+    for (int cg = 0; cg < dd->ncg_home; cg++)
     {
         if (GET_CGINFO_CONSTR(cginfo[cg]))
         {
-            for (a = dd->cgindex[cg]; a < dd->cgindex[cg+1]; a++)
+            for (int a = dd->cgindex[cg]; a < dd->cgindex[cg+1]; a++)
             {
-                a_gl = dd->gatindex[a];
+                int a_gl = dd->gatindex[a];
+                int molnr, a_mol;
+                mtopGetMolblockIndex(mtop, a_gl, &mb, &molnr, &a_mol);
 
-                gmx_mtop_atomnr_to_molblock_ind(alook, a_gl, &mb, &molnr, &a_mol);
-                molb = &mtop->molblock[mb];
+                const gmx_molblock_t *molb = &mtop->molblock[mb];
 
                 ncon1 = mtop->moltype[molb->type].ilist[F_CONSTR].nr/NRAL(F_SETTLE);
 
@@ -443,8 +426,6 @@ static void atoms_to_constraints(gmx_domdec_t *dd,
         }
     }
 
-    gmx_mtop_atomlookup_destroy(alook);
-
     if (debug)
     {
         fprintf(debug,
@@ -497,8 +478,8 @@ int dd_make_local_constraints(gmx_domdec_t *dd, int at_start,
     else
     {
         // Currently unreachable
-        at2con_mt = NULL;
-        ireq      = NULL;
+        at2con_mt = nullptr;
+        ireq      = nullptr;
     }
 
     if (dd->bInterCGsettles)
@@ -509,10 +490,10 @@ int dd_make_local_constraints(gmx_domdec_t *dd, int at_start,
     else
     {
         /* Settle works inside charge groups, we assigned them already */
-        at2settle_mt = NULL;
+        at2settle_mt = nullptr;
     }
 
-    if (at2settle_mt == NULL)
+    if (at2settle_mt == nullptr)
     {
         atoms_to_constraints(dd, mtop, cginfo, at2con_mt, nrec,
                              ilc_local, ireq);
@@ -525,7 +506,7 @@ int dd_make_local_constraints(gmx_domdec_t *dd, int at_start,
         /* Do the constraints, if present, on the first thread.
          * Do the settles on all other threads.
          */
-        t0_set = ((at2con_mt != NULL && dc->nthread > 1) ? 1 : 0);
+        t0_set = ((at2con_mt != nullptr && dc->nthread > 1) ? 1 : 0);
 
 #pragma omp parallel for num_threads(dc->nthread) schedule(static)
         for (thread = 0; thread < dc->nthread; thread++)
