@@ -49,6 +49,8 @@
 #include <stdlib.h>
 #include <string.h>
 
+#include <cassert>
+
 #include <algorithm>
 #include <string>
 
@@ -419,7 +421,7 @@ void dd_print_missing_interactions(FILE *fplog, t_commrec *cr,
 
     print_missing_interactions_atoms(fplog, cr, top_global, &top_local->idef);
     write_dd_pdb("dd_dump_err", 0, "dump", top_global, cr,
-                 -1, state_local->x, state_local->box);
+                 -1, as_rvec_array(state_local->x.data()), state_local->box);
 
     std::string errorMessage;
 
@@ -503,7 +505,7 @@ static void count_excls(const t_block *cgs, const t_blocka *excls,
     }
 }
 
-/*! \brief Run the reverse ilist generation and store it when \p bAssign = TRUE */
+/*! \brief Run the reverse ilist generation and store it in r_il when \p bAssign = TRUE */
 static int low_make_reverse_ilist(const t_ilist *il_mt, const t_atom *atom,
                                   const int * const * vsite_pbc,
                                   int *count,
@@ -555,6 +557,8 @@ static int low_make_reverse_ilist(const t_ilist *il_mt, const t_atom *atom,
                     a = ia[1+link];
                     if (bAssign)
                     {
+                        assert(r_il); //with bAssign not allowed to be null
+                        assert(r_index);
                         r_il[r_index[a]+count[a]] =
                             (ftype == F_CONSTRNC ? F_CONSTR : ftype);
                         r_il[r_index[a]+count[a]+1] = ia[0];
@@ -622,7 +626,7 @@ static int make_reverse_ilist(const t_ilist *ilist,
     snew(count, nat_mt);
     low_make_reverse_ilist(ilist, atoms->atom, vsite_pbc,
                            count,
-                           bConstr, bSettle, bBCheck, NULL, NULL,
+                           bConstr, bSettle, bBCheck, nullptr, nullptr,
                            bLinkToAllAtoms, FALSE);
 
     snew(ril_mt->index, nat_mt+1);
@@ -688,7 +692,7 @@ static gmx_reverse_top_t *make_reverse_top(const gmx_mtop_t *mtop, gmx_bool bFE,
         /* Make the atom to interaction list for this molecule type */
         nint_mt[mt] =
             make_reverse_ilist(molt->ilist, &molt->atoms,
-                               vsite_pbc_molt ? vsite_pbc_molt[mt] : NULL,
+                               vsite_pbc_molt ? vsite_pbc_molt[mt] : nullptr,
                                rt->bConstr, rt->bSettle, rt->bBCheck, FALSE,
                                &rt->ril_mt[mt]);
 
@@ -712,15 +716,15 @@ static gmx_reverse_top_t *make_reverse_top(const gmx_mtop_t *mtop, gmx_bool bFE,
     {
         t_atoms atoms_global;
 
-        rt->ril_intermol.index = NULL;
-        rt->ril_intermol.il    = NULL;
+        rt->ril_intermol.index = nullptr;
+        rt->ril_intermol.il    = nullptr;
 
         atoms_global.nr   = mtop->natoms;
-        atoms_global.atom = NULL; /* Only used with virtual sites */
+        atoms_global.atom = nullptr; /* Only used with virtual sites */
 
         *nint +=
             make_reverse_ilist(mtop->intermolecular_ilist, &atoms_global,
-                               NULL,
+                               nullptr,
                                rt->bConstr, rt->bSettle, rt->bBCheck, FALSE,
                                &rt->ril_intermol);
     }
@@ -749,7 +753,7 @@ static gmx_reverse_top_t *make_reverse_top(const gmx_mtop_t *mtop, gmx_bool bFE,
 
     rt->nthread = gmx_omp_nthreads_get(emntDomdec);
     snew(rt->th_work, rt->nthread);
-    if (vsite_pbc_molt != NULL)
+    if (vsite_pbc_molt != nullptr)
     {
         for (thread = 0; thread < rt->nthread; thread++)
         {
@@ -778,7 +782,7 @@ void dd_make_reverse_top(FILE *fplog,
      */
 
     dd->reverse_top = make_reverse_top(mtop, ir->efep != efepNO,
-                                       vsite ? vsite->vsite_pbc_molt : NULL,
+                                       vsite ? vsite->vsite_pbc_molt : nullptr,
                                        !dd->bInterCGcons, !dd->bInterCGsettles,
                                        bBCheck, &dd->nbonded_global);
 
@@ -1223,7 +1227,7 @@ static void combine_idef(t_idef *dest, const thread_work_t *src, int nsrc,
             int      nral1 = 0, ftv = 0;
 
             vpbc = ((interaction_function[ftype].flags & IF_VSITE) &&
-                    vsite->vsite_pbc_loc != NULL);
+                    vsite->vsite_pbc_loc != nullptr);
             if (vpbc)
             {
                 nral1 = 1 + NRAL(ftype);
@@ -2021,8 +2025,8 @@ static int make_local_bondeds_excls(gmx_domdec_t *dd,
                 }
                 else
                 {
-                    vsite_pbc        = NULL;
-                    vsite_pbc_nalloc = NULL;
+                    vsite_pbc        = nullptr;
+                    vsite_pbc_nalloc = nullptr;
                 }
 
                 rt->th_work[thread].nbonded =
@@ -2133,7 +2137,7 @@ void dd_make_local_top(gmx_domdec_t *dd, gmx_domdec_zones_t *zones,
     real     rc = -1;
     ivec     rcheck;
     int      d, nexcl;
-    t_pbc    pbc, *pbc_null = NULL;
+    t_pbc    pbc, *pbc_null = nullptr;
 
     if (debug)
     {
@@ -2191,7 +2195,7 @@ void dd_make_local_top(gmx_domdec_t *dd, gmx_domdec_zones_t *zones,
             }
             else
             {
-                pbc_null = NULL;
+                pbc_null = nullptr;
             }
         }
     }
@@ -2246,7 +2250,7 @@ gmx_localtop_t *dd_init_local_top(const gmx_mtop_t *top_global)
 
     for (i = 0; i < F_NRE; i++)
     {
-        top->idef.il[i].iatoms = NULL;
+        top->idef.il[i].iatoms = nullptr;
         top->idef.il[i].nalloc = 0;
     }
     top->idef.ilsort   = ilsortUNKNOWN;
@@ -2265,11 +2269,12 @@ void dd_init_local_state(gmx_domdec_t *dd,
         buf[1] = state_global->ngtc;
         buf[2] = state_global->nnhpres;
         buf[3] = state_global->nhchainlength;
-        buf[4] = state_global->dfhist.nlambda;
+        buf[4] = state_global->dfhist ? state_global->dfhist->nlambda : 0;
     }
     dd_bcast(dd, NITEM_DD_INIT_LOCAL_STATE*sizeof(int), buf);
 
-    init_state(state_local, 0, buf[1], buf[2], buf[3], buf[4]);
+    init_gtc_state(state_local, buf[1], buf[2], buf[3]);
+    init_dfhist_state(state_local, buf[4]);
     state_local->flags = buf[0];
 }
 
@@ -2351,16 +2356,16 @@ t_blocka *make_charge_group_links(const gmx_mtop_t *mtop, gmx_domdec_t *dd,
         t_atoms atoms;
 
         atoms.nr   = mtop->natoms;
-        atoms.atom = NULL;
+        atoms.atom = nullptr;
 
         make_reverse_ilist(mtop->intermolecular_ilist, &atoms,
-                           NULL, FALSE, FALSE, FALSE, TRUE, &ril_intermol);
+                           nullptr, FALSE, FALSE, FALSE, TRUE, &ril_intermol);
     }
 
     snew(link, 1);
     snew(link->index, ncg_mtop(mtop)+1);
     link->nalloc_a = 0;
-    link->a        = NULL;
+    link->a        = nullptr;
 
     link->index[0] = 0;
     cg_offset      = 0;
@@ -2381,7 +2386,7 @@ t_blocka *make_charge_group_links(const gmx_mtop_t *mtop, gmx_domdec_t *dd,
          * The constraints are discarded here.
          */
         make_reverse_ilist(molt->ilist, &molt->atoms,
-                           NULL, FALSE, FALSE, FALSE, TRUE, &ril);
+                           nullptr, FALSE, FALSE, FALSE, TRUE, &ril);
 
         cgi_mb = &cginfo_mb[mb];
 
@@ -2585,7 +2590,7 @@ static void bonded_cg_distance_mol(gmx_moltype_t *molt, int *at2cg,
 /*! \brief Set the distance, function type and atom indices for the longest atom distance involved in intermolecular interactions for two-body and multi-body bonded interactions */
 static void bonded_distance_intermol(const t_ilist *ilists_intermol,
                                      gmx_bool bBCheck,
-                                     const rvec *x, int ePBC, matrix box,
+                                     const rvec *x, int ePBC, const matrix box,
                                      bonded_distance_t *bd_2b,
                                      bonded_distance_t *bd_mb)
 {
@@ -2630,18 +2635,33 @@ static void bonded_distance_intermol(const t_ilist *ilists_intermol,
     }
 }
 
+//! Returns whether \p molt has at least one virtual site
+static bool moltypeHasVsite(const gmx_moltype_t &molt)
+{
+    bool hasVsite = false;
+    for (int i = 0; i < F_NRE; i++)
+    {
+        if ((interaction_function[i].flags & IF_VSITE) &&
+            molt.ilist[i].nr > 0)
+        {
+            hasVsite = true;
+        }
+    }
+
+    return hasVsite;
+}
+
 //! Compute charge group centers of mass for molecule \p molt
 static void get_cgcm_mol(const gmx_moltype_t *molt,
                          const gmx_ffparams_t *ffparams,
-                         int ePBC, t_graph *graph, matrix box,
-                         const gmx_vsite_t *vsite,
+                         int ePBC, t_graph *graph, const matrix box,
                          const rvec *x, rvec *xs, rvec *cg_cm)
 {
     int n, i;
 
     if (ePBC != epbcNONE)
     {
-        mk_mshift(NULL, graph, ePBC, box, x);
+        mk_mshift(nullptr, graph, ePBC, box, x);
 
         shift_x(graph, box, x, xs);
         /* By doing an extra mk_mshift the molecules that are broken
@@ -2649,7 +2669,7 @@ static void get_cgcm_mol(const gmx_moltype_t *molt,
          * will be made whole again. Such are the healing powers
          * of GROMACS.
          */
-        mk_mshift(NULL, graph, ePBC, box, xs);
+        mk_mshift(nullptr, graph, ePBC, box, xs);
     }
     else
     {
@@ -2664,46 +2684,26 @@ static void get_cgcm_mol(const gmx_moltype_t *molt,
         }
     }
 
-    if (vsite)
+    if (moltypeHasVsite(*molt))
     {
-        construct_vsites(vsite, xs, 0.0, NULL,
+        construct_vsites(nullptr, xs, 0.0, nullptr,
                          ffparams->iparams, molt->ilist,
-                         epbcNONE, TRUE, NULL, NULL);
+                         epbcNONE, TRUE, nullptr, nullptr);
     }
 
-    calc_cgcm(NULL, 0, molt->cgs.nr, &molt->cgs, xs, cg_cm);
-}
-
-//! Returns whether \p molt has a virtual site
-static int have_vsite_molt(gmx_moltype_t *molt)
-{
-    int      i;
-    gmx_bool bVSite;
-
-    bVSite = FALSE;
-    for (i = 0; i < F_NRE; i++)
-    {
-        if ((interaction_function[i].flags & IF_VSITE) &&
-            molt->ilist[i].nr > 0)
-        {
-            bVSite = TRUE;
-        }
-    }
-
-    return bVSite;
+    calc_cgcm(nullptr, 0, molt->cgs.nr, &molt->cgs, xs, cg_cm);
 }
 
 void dd_bonded_cg_distance(FILE *fplog,
                            const gmx_mtop_t *mtop,
                            const t_inputrec *ir,
-                           const rvec *x, matrix box,
+                           const rvec *x, const matrix box,
                            gmx_bool bBCheck,
                            real *r_2b, real *r_mb)
 {
     gmx_bool           bExclRequired;
     int                mb, at_offset, *at2cg, mol;
     t_graph            graph;
-    gmx_vsite_t       *vsite;
     gmx_molblock_t    *molb;
     gmx_moltype_t     *molt;
     rvec              *xs, *cg_cm;
@@ -2712,8 +2712,6 @@ void dd_bonded_cg_distance(FILE *fplog,
 
     bExclRequired = inputrecExclForces(ir);
 
-    vsite = init_vsite(mtop, NULL, TRUE);
-
     *r_2b     = 0;
     *r_mb     = 0;
     at_offset = 0;
@@ -2729,7 +2727,7 @@ void dd_bonded_cg_distance(FILE *fplog,
         {
             if (ir->ePBC != epbcNONE)
             {
-                mk_graph_ilist(NULL, molt->ilist, 0, molt->atoms.nr, FALSE, FALSE,
+                mk_graph_ilist(nullptr, molt->ilist, 0, molt->atoms.nr, FALSE, FALSE,
                                &graph);
             }
 
@@ -2739,7 +2737,6 @@ void dd_bonded_cg_distance(FILE *fplog,
             for (mol = 0; mol < molb->nmol; mol++)
             {
                 get_cgcm_mol(molt, &mtop->ffparams, ir->ePBC, &graph, box,
-                             have_vsite_molt(molt) ? vsite : NULL,
                              x+at_offset, xs, cg_cm);
 
                 bonded_distance_t bd_mol_2b = { 0, -1, -1, -1 };
@@ -2770,9 +2767,6 @@ void dd_bonded_cg_distance(FILE *fplog,
         }
     }
 
-    /* We should have a vsite free routine, but here we can simply free */
-    sfree(vsite);
-
     if (mtop->bIntermolecularInteractions)
     {
         if (ncg_mtop(mtop) < mtop->natoms)
