@@ -3,7 +3,7 @@
  *
  * Copyright (c) 1991-2000, University of Groningen, The Netherlands.
  * Copyright (c) 2001-2004, The GROMACS development team.
- * Copyright (c) 2013,2014,2015, by the GROMACS development team, led by
+ * Copyright (c) 2013,2014,2015,2016,2017, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -48,12 +48,24 @@
 #ifndef GMX_ESSENTIALDYNAMICS_EDSAM_H
 #define GMX_ESSENTIALDYNAMICS_EDSAM_H
 
-#include "gromacs/fileio/filenm.h"
-#include "gromacs/legacyheaders/typedefs.h"
+#include "gromacs/math/vectypes.h"
+#include "gromacs/utility/basedefinitions.h"
 
-#ifdef __cplusplus
-extern "C" {
-#endif
+
+/*! \brief Abstract type for essential dynamics
+ *
+ * The main type is defined only in edsam.cpp
+ */
+typedef struct gmx_edsam *gmx_edsam_t;
+
+struct gmx_domdec_t;
+struct gmx_mtop_t;
+struct gmx_output_env_t;
+struct ObservablesHistory;
+struct t_commrec;
+struct t_filenm;
+struct t_inputrec;
+class t_state;
 
 /*! \brief Applies essential dynamics constrains as defined in the .edi input file.
  *
@@ -65,45 +77,36 @@ extern "C" {
  * \param box               The simulation box.
  * \param ed                The essential dynamics data.
  */
-void do_edsam(t_inputrec *ir, gmx_int64_t step,
+void do_edsam(const t_inputrec *ir, gmx_int64_t step,
               t_commrec *cr, rvec xs[], rvec v[], matrix box, gmx_edsam_t ed);
 
 
-/*! \brief Reads in the .edi file containing the essential dynamics and flooding data.
- *
- * This function opens the ED input and output files, reads in all datasets it finds
- * in the input file, and cross-checks whether the .edi file information is consistent
- * with the essential dynamics data found in the checkpoint file (if present).
- * gmx make_edi can be used to create an .edi input file.
- *
- * \param natoms            Number of atoms of the whole MD system.
- * \param EDstate           Essential dynamics and flooding data stored in the checkpoint file.
- * \param nfile             Number of entries (files) in the fnm structure.
- * \param fnm               The filenames struct; it contains also the names of the
- *                          essential dynamics and flooding in + output files.
- * \param Flags             Flags passed over from main, used to determine
- *                          whether we are appending.
- * \param oenv              Needed to open the output xvgr file.
- * \param cr                Data needed for MPI communication.
- * \returns                 Pointer to the initialized essential dynamics / flooding data.
- */
-gmx_edsam_t ed_open(int natoms, edsamstate_t *EDstate, int nfile, const t_filenm fnm[],
-                    unsigned long Flags, const output_env_t oenv, t_commrec *cr);
-
-
 /*! \brief Initializes the essential dynamics and flooding module.
  *
+ * \param ediFileName       Essential dynamics input file.
+ * \param edoFileName       Output file for essential dynamics data.
  * \param mtop              Molecular topology.
  * \param ir                MD input parameter record.
  * \param cr                Data needed for MPI communication.
- * \param ed                The essential dynamics data.
- * \param x                 Positions of the whole MD system.
- * \param box               The simulation box.
- * \param EDstate           ED data stored in the checkpoint file.
+ * \param constr            Data structure keeping the constraint information.
+ * \param globalState       The global state, only used on the master rank.
+ * \param oh                The observables history container.
+ * \param oenv              The output environment information.
+ * \param bAppend           Append to existing output files?
+ *
+ * \returns                 A pointer to the ED data structure.
  */
-void init_edsam(gmx_mtop_t *mtop, t_inputrec *ir, t_commrec *cr,
-                gmx_edsam_t ed, rvec x[], matrix box, edsamstate_t *EDstate);
-
+gmx_edsam_t init_edsam(
+        const char             *ediFileName,
+        const char             *edoFileName,
+        const gmx_mtop_t       *mtop,
+        const t_inputrec       *ir,
+        t_commrec              *cr,
+        struct gmx_constr      *constr,
+        const t_state          *globalState,
+        ObservablesHistory     *oh,
+        const gmx_output_env_t *oenv,
+        gmx_bool                bAppend);
 
 /*! \brief Make a selection of the home atoms for the ED groups.
  *
@@ -126,7 +129,7 @@ void dd_make_local_ed_indices(gmx_domdec_t *dd, gmx_edsam_t ed);
  * \param step              Number of the time step.
  * \param bNS               Are we in a neighbor searching step?
  */
-void do_flood(t_commrec *cr, t_inputrec *ir, rvec x[], rvec force[], gmx_edsam_t ed,
+void do_flood(t_commrec *cr, const t_inputrec *ir, rvec x[], rvec force[], gmx_edsam_t ed,
               matrix box, gmx_int64_t step, gmx_bool bNS);
 
 /*! \brief Clean up
@@ -135,8 +138,4 @@ void do_flood(t_commrec *cr, t_inputrec *ir, rvec x[], rvec force[], gmx_edsam_t
  */
 void done_ed(gmx_edsam_t *ed);
 
-#ifdef __cplusplus
-}
-#endif
-
 #endif
