@@ -47,6 +47,7 @@
 #include "gromacs/math/vec.h"
 #include "gromacs/simd/simd.h"
 #include "gromacs/simd/simd_math.h"
+#include "gromacs/utility/arrayref.h"
 #include "gromacs/utility/exceptions.h"
 #include "gromacs/utility/smalloc.h"
 
@@ -154,27 +155,31 @@ void pme_init_all_work(struct pme_solve_work_t **work, int nthread, int nkx)
 
 static void free_work(struct pme_solve_work_t *work)
 {
-    sfree(work->mhx);
-    sfree(work->mhy);
-    sfree(work->mhz);
-    sfree(work->m2);
-    sfree_aligned(work->denom);
-    sfree_aligned(work->tmp1);
-    sfree_aligned(work->tmp2);
-    sfree_aligned(work->eterm);
-    sfree(work->m2inv);
+    if (work)
+    {
+        sfree(work->mhx);
+        sfree(work->mhy);
+        sfree(work->mhz);
+        sfree(work->m2);
+        sfree_aligned(work->denom);
+        sfree_aligned(work->tmp1);
+        sfree_aligned(work->tmp2);
+        sfree_aligned(work->eterm);
+        sfree(work->m2inv);
+    }
 }
 
 void pme_free_all_work(struct pme_solve_work_t **work, int nthread)
 {
-    int thread;
-
-    for (thread = 0; thread < nthread; thread++)
+    if (*work)
     {
-        free_work(&(*work)[thread]);
+        for (int thread = 0; thread < nthread; thread++)
+        {
+            free_work(&(*work)[thread]);
+        }
     }
-    sfree(work);
-    *work = NULL;
+    sfree(*work);
+    *work = nullptr;
 }
 
 void get_pme_ener_vir_q(struct pme_solve_work_t *work, int nthread,
@@ -215,30 +220,33 @@ void get_pme_ener_vir_lj(struct pme_solve_work_t *work, int nthread,
 
 #if defined PME_SIMD_SOLVE
 /* Calculate exponentials through SIMD */
-gmx_inline static void calc_exponentials_q(int gmx_unused start, int end, real f, real *d_aligned, real *r_aligned, real *e_aligned)
+gmx_inline static void calc_exponentials_q(int, int, real f, ArrayRef<const SimdReal> d_aligned, ArrayRef<const SimdReal> r_aligned, ArrayRef<SimdReal> e_aligned)
 {
     {
         SimdReal              f_simd(f);
         SimdReal              tmp_d1, tmp_r, tmp_e;
-        int                   kx;
 
         /* We only need to calculate from start. But since start is 0 or 1
          * and we want to use aligned loads/stores, we always start from 0.
          */
-        for (kx = 0; kx < end; kx += GMX_SIMD_REAL_WIDTH)
+        GMX_ASSERT(d_aligned.size() == r_aligned.size(), "d and r must have same size");
+        GMX_ASSERT(d_aligned.size() == e_aligned.size(), "d and e must have same size");
+        for (size_t kx = 0; kx != d_aligned.size(); ++kx)
         {
-            tmp_d1   = load(d_aligned+kx);
-            tmp_r    = load(r_aligned+kx);
-            tmp_r    = gmx::exp(tmp_r);
-            tmp_e    = f_simd / tmp_d1;
-            tmp_e    = tmp_e * tmp_r;
-            store(e_aligned+kx, tmp_e);
+            tmp_d1        = d_aligned[kx];
+            tmp_r         = r_aligned[kx];
+            tmp_r         = gmx::exp(tmp_r);
+            tmp_e         = f_simd / tmp_d1;
+            tmp_e         = tmp_e * tmp_r;
+            e_aligned[kx] = tmp_e;
         }
     }
 }
 #else
-gmx_inline static void calc_exponentials_q(int start, int end, real f, real *d, real *r, real *e)
+gmx_inline static void calc_exponentials_q(int start, int end, real f, ArrayRef<real> d, ArrayRef<real> r, ArrayRef<real> e)
 {
+    GMX_ASSERT(d.size() == r.size(), "d and r must have same size");
+    GMX_ASSERT(d.size() == e.size(), "d and e must have same size");
     int kx;
     for (kx = start; kx < end; kx++)
     {
@@ -257,32 +265,36 @@ gmx_inline static void calc_exponentials_q(int start, int end, real f, real *d,
 
 #if defined PME_SIMD_SOLVE
 /* Calculate exponentials through SIMD */
-gmx_inline static void calc_exponentials_lj(int gmx_unused start, int end, real *r_aligned, real *factor_aligned, real *d_aligned)
+gmx_inline static void calc_exponentials_lj(int, int, ArrayRef<SimdReal> r_aligned, ArrayRef<SimdReal> factor_aligned, ArrayRef<SimdReal> d_aligned)
 {
     SimdReal              tmp_r, tmp_d, tmp_fac, d_inv, tmp_mk;
     const SimdReal        sqr_PI = sqrt(SimdReal(M_PI));
-    int                   kx;
-    for (kx = 0; kx < end; kx += GMX_SIMD_REAL_WIDTH)
+
+    GMX_ASSERT(d_aligned.size() == r_aligned.size(), "d and r must have same size");
+    GMX_ASSERT(d_aligned.size() == factor_aligned.size(), "d and factor must have same size");
+    for (size_t kx = 0; kx != d_aligned.size(); ++kx)
     {
         /* We only need to calculate from start. But since start is 0 or 1
          * and we want to use aligned loads/stores, we always start from 0.
          */
-        tmp_d = load(d_aligned+kx);
-        d_inv = SimdReal(1.0) / tmp_d;
-        store(d_aligned+kx, d_inv);
-        tmp_r = load(r_aligned+kx);
-        tmp_r = gmx::exp(tmp_r);
-        store(r_aligned+kx, tmp_r);
-        tmp_mk  = load(factor_aligned+kx);
-        tmp_fac = sqr_PI * tmp_mk * erfc(tmp_mk);
-        store(factor_aligned+kx, tmp_fac);
+        tmp_d              = d_aligned[kx];
+        d_inv              = SimdReal(1.0) / tmp_d;
+        d_aligned[kx]      = d_inv;
+        tmp_r              = r_aligned[kx];
+        tmp_r              = gmx::exp(tmp_r);
+        r_aligned[kx]      = tmp_r;
+        tmp_mk             = factor_aligned[kx];
+        tmp_fac            = sqr_PI * tmp_mk * erfc(tmp_mk);
+        factor_aligned[kx] = tmp_fac;
     }
 }
 #else
-gmx_inline static void calc_exponentials_lj(int start, int end, real *r, real *tmp2, real *d)
+gmx_inline static void calc_exponentials_lj(int start, int end, ArrayRef<real> r, ArrayRef<real> tmp2, ArrayRef<real> d)
 {
     int  kx;
     real mk;
+    GMX_ASSERT(d.size() == r.size(), "d and r must have same size");
+    GMX_ASSERT(d.size() == tmp2.size(), "d and tmp2 must have same size");
     for (kx = start; kx < end; kx++)
     {
         d[kx] = 1.0/d[kx];
@@ -301,8 +313,13 @@ gmx_inline static void calc_exponentials_lj(int start, int end, real *r, real *t
 }
 #endif
 
-int solve_pme_yzx(struct gmx_pme_t *pme, t_complex *grid,
-                  real ewaldcoeff, real vol,
+#if defined PME_SIMD_SOLVE
+using PME_T = SimdReal;
+#else
+using PME_T = real;
+#endif
+
+int solve_pme_yzx(const gmx_pme_t *pme, t_complex *grid, real vol,
                   gmx_bool bEnerVir,
                   int nthread, int thread)
 {
@@ -312,7 +329,8 @@ int solve_pme_yzx(struct gmx_pme_t *pme, t_complex *grid,
     int                      kx, ky, kz, maxkx, maxky;
     int                      nx, ny, nz, iyz0, iyz1, iyz, iy, iz, kxstart, kxend;
     real                     mx, my, mz;
-    real                     factor = M_PI*M_PI/(ewaldcoeff*ewaldcoeff);
+    real                     ewaldcoeff = pme->ewaldcoeff_q;
+    real                     factor     = M_PI*M_PI/(ewaldcoeff*ewaldcoeff);
     real                     ets2, struct2, vfactor, ets2vf;
     real                     d1, d2, energy = 0;
     real                     by, bz;
@@ -346,6 +364,8 @@ int solve_pme_yzx(struct gmx_pme_t *pme, t_complex *grid,
     rzy = pme->recipbox[ZZ][YY];
     rzz = pme->recipbox[ZZ][ZZ];
 
+    GMX_ASSERT(rxx != 0.0, "Someone broke the reciprocal box again");
+
     maxkx = (nx+1)/2;
     maxky = (ny+1)/2;
 
@@ -454,7 +474,10 @@ int solve_pme_yzx(struct gmx_pme_t *pme, t_complex *grid,
                 m2inv[kx] = 1.0/m2[kx];
             }
 
-            calc_exponentials_q(kxstart, kxend, elfac, denom, tmp1, eterm);
+            calc_exponentials_q(kxstart, kxend, elfac,
+                                ArrayRef<PME_T>(denom, denom+roundUpToMultipleOfFactor<c_simdWidth>(kxend)),
+                                ArrayRef<PME_T>(tmp1, tmp1+roundUpToMultipleOfFactor<c_simdWidth>(kxend)),
+                                ArrayRef<PME_T>(eterm, eterm+roundUpToMultipleOfFactor<c_simdWidth>(kxend)));
 
             for (kx = kxstart; kx < kxend; kx++, p0++)
             {
@@ -516,7 +539,11 @@ int solve_pme_yzx(struct gmx_pme_t *pme, t_complex *grid,
                 tmp1[kx]  = -factor*m2k;
             }
 
-            calc_exponentials_q(kxstart, kxend, elfac, denom, tmp1, eterm);
+            calc_exponentials_q(kxstart, kxend, elfac,
+                                ArrayRef<PME_T>(denom, denom+roundUpToMultipleOfFactor<c_simdWidth>(kxend)),
+                                ArrayRef<PME_T>(tmp1, tmp1+roundUpToMultipleOfFactor<c_simdWidth>(kxend)),
+                                ArrayRef<PME_T>(eterm, eterm+roundUpToMultipleOfFactor<c_simdWidth>(kxend)));
+
 
             for (kx = kxstart; kx < kxend; kx++, p0++)
             {
@@ -552,8 +579,7 @@ int solve_pme_yzx(struct gmx_pme_t *pme, t_complex *grid,
     return local_ndata[YY]*local_ndata[XX];
 }
 
-int solve_pme_lj_yzx(struct gmx_pme_t *pme, t_complex **grid, gmx_bool bLB,
-                     real ewaldcoeff, real vol,
+int solve_pme_lj_yzx(const gmx_pme_t *pme, t_complex **grid, gmx_bool bLB, real vol,
                      gmx_bool bEnerVir, int nthread, int thread)
 {
     /* do recip sum over local cells in grid */
@@ -562,7 +588,8 @@ int solve_pme_lj_yzx(struct gmx_pme_t *pme, t_complex **grid, gmx_bool bLB,
     int                      kx, ky, kz, maxkx, maxky;
     int                      nx, ny, nz, iy, iyz0, iyz1, iyz, iz, kxstart, kxend;
     real                     mx, my, mz;
-    real                     factor = M_PI*M_PI/(ewaldcoeff*ewaldcoeff);
+    real                     ewaldcoeff = pme->ewaldcoeff_lj;
+    real                     factor     = M_PI*M_PI/(ewaldcoeff*ewaldcoeff);
     real                     ets2, ets2vf;
     real                     eterm, vterm, d1, d2, energy = 0;
     real                     by, bz;
@@ -690,7 +717,10 @@ int solve_pme_lj_yzx(struct gmx_pme_t *pme, t_complex **grid, gmx_bool bLB,
                 tmp2[kx] = 0;
             }
 
-            calc_exponentials_lj(kxstart, kxend, tmp1, tmp2, denom);
+            calc_exponentials_lj(kxstart, kxend,
+                                 ArrayRef<PME_T>(tmp1, tmp1+roundUpToMultipleOfFactor<c_simdWidth>(kxend)),
+                                 ArrayRef<PME_T>(tmp2, tmp2+roundUpToMultipleOfFactor<c_simdWidth>(kxend)),
+                                 ArrayRef<PME_T>(denom, denom+roundUpToMultipleOfFactor<c_simdWidth>(kxend)));
 
             for (kx = kxstart; kx < kxend; kx++)
             {
@@ -830,7 +860,10 @@ int solve_pme_lj_yzx(struct gmx_pme_t *pme, t_complex **grid, gmx_bool bLB,
                 tmp2[kx] = 0;
             }
 
-            calc_exponentials_lj(kxstart, kxend, tmp1, tmp2, denom);
+            calc_exponentials_lj(kxstart, kxend,
+                                 ArrayRef<PME_T>(tmp1, tmp1+roundUpToMultipleOfFactor<c_simdWidth>(kxend)),
+                                 ArrayRef<PME_T>(tmp2, tmp2+roundUpToMultipleOfFactor<c_simdWidth>(kxend)),
+                                 ArrayRef<PME_T>(denom, denom+roundUpToMultipleOfFactor<c_simdWidth>(kxend)));
 
             for (kx = kxstart; kx < kxend; kx++)
             {
