@@ -2,7 +2,7 @@
  * This file is part of the GROMACS molecular simulation package.
  *
  * Copyright (c) 1991-2003 David van der Spoel, Erik Lindahl, University of Groningen.
- * Copyright (c) 2013,2014,2015, by the GROMACS development team, led by
+ * Copyright (c) 2013,2014,2015,2016,2017, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -36,10 +36,12 @@
 #include "gmxpre.h"
 
 #include <errno.h>
-#include <math.h>
+#include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
+#include <cmath>
+
 #include "external/fftpack/fftpack.h"
 
 #include "gromacs/fft/fft.h"
@@ -70,10 +72,6 @@ struct gmx_fft
     real  *         work;     /**< 1st 4n reserved for cfft, 1st 2n for rfft */
 };
 
-#include <math.h>
-#include <stdio.h>
-
-
 int
 gmx_fft_init_1d(gmx_fft_t *        pfft,
                 int                nx,
@@ -81,23 +79,23 @@ gmx_fft_init_1d(gmx_fft_t *        pfft,
 {
     gmx_fft_t    fft;
 
-    if (pfft == NULL)
+    if (pfft == nullptr)
     {
         gmx_fatal(FARGS, "Invalid FFT opaque type pointer.");
         return EINVAL;
     }
-    *pfft = NULL;
+    *pfft = nullptr;
 
-    if ( (fft = (struct gmx_fft *)malloc(sizeof(struct gmx_fft))) == NULL)
+    if ( (fft = (struct gmx_fft *)malloc(sizeof(struct gmx_fft))) == nullptr)
     {
         return ENOMEM;
     }
 
-    fft->next = NULL;
+    fft->next = nullptr;
     fft->n    = nx;
 
     /* Need 4*n storage for 1D complex FFT */
-    if ( (fft->work = (real *)malloc(sizeof(real)*(4*nx))) == NULL)
+    if ( (fft->work = (real *)malloc(sizeof(real)*(4*nx))) == nullptr)
     {
         free(fft);
         return ENOMEM;
@@ -121,23 +119,23 @@ gmx_fft_init_1d_real(gmx_fft_t *        pfft,
 {
     gmx_fft_t    fft;
 
-    if (pfft == NULL)
+    if (pfft == nullptr)
     {
         gmx_fatal(FARGS, "Invalid FFT opaque type pointer.");
         return EINVAL;
     }
-    *pfft = NULL;
+    *pfft = nullptr;
 
-    if ( (fft = (struct gmx_fft *)malloc(sizeof(struct gmx_fft))) == NULL)
+    if ( (fft = (struct gmx_fft *)malloc(sizeof(struct gmx_fft))) == nullptr)
     {
         return ENOMEM;
     }
 
-    fft->next = NULL;
+    fft->next = nullptr;
     fft->n    = nx;
 
     /* Need 2*n storage for 1D real FFT */
-    if ((fft->work = (real *)malloc(sizeof(real)*(2*nx))) == NULL)
+    if ((fft->work = (real *)malloc(sizeof(real)*(2*nx))) == nullptr)
     {
         free(fft);
         return ENOMEM;
@@ -162,15 +160,15 @@ gmx_fft_init_2d_real(gmx_fft_t *        pfft,
     int           nyc = (ny/2 + 1);
     int           rc;
 
-    if (pfft == NULL)
+    if (pfft == nullptr)
     {
         gmx_fatal(FARGS, "Invalid FFT opaque type pointer.");
         return EINVAL;
     }
-    *pfft = NULL;
+    *pfft = nullptr;
 
     /* Create the X transform */
-    if ( (fft = (struct gmx_fft *)malloc(sizeof(struct gmx_fft))) == NULL)
+    if ( (fft = (struct gmx_fft *)malloc(sizeof(struct gmx_fft))) == nullptr)
     {
         return ENOMEM;
     }
@@ -180,7 +178,7 @@ gmx_fft_init_2d_real(gmx_fft_t *        pfft,
     /* Need 4*nx storage for 1D complex FFT, and another
      * 2*nx*nyc elements for complex-to-real storage in our high-level routine.
      */
-    if ( (fft->work = (real *)malloc(sizeof(real)*(4*nx+2*nx*nyc))) == NULL)
+    if ( (fft->work = (real *)malloc(sizeof(real)*(4*nx+2*nx*nyc))) == nullptr)
     {
         free(fft);
         return ENOMEM;
@@ -391,7 +389,7 @@ gmx_fft_2d_real          (gmx_fft_t                  fft,
                 }
             }
         }
-        data = (t_complex *)out_data;
+        data = static_cast<t_complex *>(out_data);
 
         /* y real-to-complex FFTs */
         for (i = 0; i < nx; i++)
@@ -425,12 +423,12 @@ gmx_fft_2d_real          (gmx_fft_t                  fft,
         if (in_data != out_data)
         {
             memcpy(work, in_data, sizeof(t_complex)*nx*nyc);
-            data = (t_complex *)work;
+            data = reinterpret_cast<t_complex *>(work);
         }
         else
         {
             /* in-place */
-            data = (t_complex *)out_data;
+            data = reinterpret_cast<t_complex *>(out_data);
         }
 
         /* Transpose to get X arrays */
@@ -480,10 +478,10 @@ gmx_fft_2d_real          (gmx_fft_t                  fft,
 void
 gmx_fft_destroy(gmx_fft_t      fft)
 {
-    if (fft != NULL)
+    if (fft != nullptr)
     {
         free(fft->work);
-        if (fft->next != NULL)
+        if (fft->next != nullptr)
         {
             gmx_fft_destroy(fft->next);
         }
