@@ -3,7 +3,7 @@
  *
  * Copyright (c) 1991-2000, University of Groningen, The Netherlands.
  * Copyright (c) 2001-2004, The GROMACS development team.
- * Copyright (c) 2013,2014,2015,2016, by the GROMACS development team, led by
+ * Copyright (c) 2013,2014,2015,2016,2017, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -47,11 +47,13 @@
 extern "C" {
 #endif
 
+class energyhistory_t;
 struct gmx_file_position_t;
+struct ObservablesHistory;
 struct t_commrec;
 struct t_fileio;
 struct t_inputrec;
-struct t_state;
+class t_state;
 struct t_trxframe;
 
 /* the name of the environment variable to disable fsync failure checks with */
@@ -67,12 +69,12 @@ void write_checkpoint(const char *fn, gmx_bool bNumberAndKeep,
                       int eIntegrator, int simulation_part,
                       gmx_bool bExpanded, int elamstats,
                       gmx_int64_t step, double t,
-                      t_state *state);
+                      t_state *state, ObservablesHistory *observablesHistory);
 
 /* Loads a checkpoint from fn for run continuation.
  * Generates a fatal error on system size mismatch.
  * The master node reads the file
- * and communicates all the modified number of steps and the parallel setup,
+ * and communicates all the modified number of steps,
  * but not the state itself.
  * When bAppend is set, lock the log file and truncate the existing output
  * files so they can be appended.
@@ -81,9 +83,10 @@ void write_checkpoint(const char *fn, gmx_bool bNumberAndKeep,
  * With reproducibilityRequested warns about version, build, #ranks differences.
  */
 void load_checkpoint(const char *fn, FILE **fplog,
-                     const t_commrec *cr, ivec dd_nc, int *npme,
+                     const t_commrec *cr, const ivec dd_nc,
                      t_inputrec *ir, t_state *state,
                      gmx_bool *bReadEkin,
+                     ObservablesHistory *observablesHistory,
                      gmx_bool bAppend, gmx_bool bForceAppend,
                      gmx_bool reproducibilityRequested);
 
