@@ -3,7 +3,7 @@
  *
  * Copyright (c) 1991-2000, University of Groningen, The Netherlands.
  * Copyright (c) 2001-2004, The GROMACS development team.
- * Copyright (c) 2013,2014,2015,2016, by the GROMACS development team, led by
+ * Copyright (c) 2013,2014,2015,2016,2017, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -50,7 +50,6 @@
 #include "gromacs/fileio/tpxio.h"
 #include "gromacs/fileio/trxio.h"
 #include "gromacs/math/vec.h"
-#include "gromacs/topology/atomprop.h"
 #include "gromacs/topology/atoms.h"
 #include "gromacs/topology/block.h"
 #include "gromacs/topology/mtop_util.h"
@@ -59,6 +58,7 @@
 #include "gromacs/trajectory/trajectoryframe.h"
 #include "gromacs/utility/cstringutil.h"
 #include "gromacs/utility/fatalerror.h"
+#include "gromacs/utility/gmxassert.h"
 #include "gromacs/utility/smalloc.h"
 
 void write_sto_conf_indexed(const char *outfile, const char *title,
@@ -80,8 +80,6 @@ void write_sto_conf_indexed(const char *outfile, const char *title,
             break;
         case efG96:
             clear_trxframe(&fr, TRUE);
-            fr.bTitle = TRUE;
-            fr.title  = title;
             fr.natoms = atoms->nr;
             fr.bAtoms = TRUE;
             fr.atoms  = const_cast<t_atoms *>(atoms);
@@ -95,7 +93,7 @@ void write_sto_conf_indexed(const char *outfile, const char *title,
             fr.bBox = TRUE;
             copy_mat(box, fr.box);
             out = gmx_fio_fopen(outfile, "w");
-            write_g96_conf(out, &fr, nindex, index);
+            write_g96_conf(out, title, &fr, nindex, index);
             gmx_fio_fclose(out);
             break;
         case efPDB:
@@ -103,7 +101,7 @@ void write_sto_conf_indexed(const char *outfile, const char *title,
         case efENT:
         case efPQR:
             out = gmx_fio_fopen(outfile, "w");
-            write_pdbfile_indexed(out, title, atoms, x, ePBC, box, ' ', -1, nindex, index, NULL, TRUE);
+            write_pdbfile_indexed(out, title, atoms, x, ePBC, box, ' ', -1, nindex, index, nullptr, TRUE);
             gmx_fio_fclose(out);
             break;
         case efESP:
@@ -134,8 +132,6 @@ void write_sto_conf(const char *outfile, const char *title, const t_atoms *atoms
             break;
         case efG96:
             clear_trxframe(&fr, TRUE);
-            fr.bTitle = TRUE;
-            fr.title  = title;
             fr.natoms = atoms->nr;
             fr.bAtoms = TRUE;
             fr.atoms  = const_cast<t_atoms *>(atoms); // TODO check
@@ -149,19 +145,19 @@ void write_sto_conf(const char *outfile, const char *title, const t_atoms *atoms
             fr.bBox = TRUE;
             copy_mat(box, fr.box);
             out = gmx_fio_fopen(outfile, "w");
-            write_g96_conf(out, &fr, -1, NULL);
+            write_g96_conf(out, title, &fr, -1, nullptr);
             gmx_fio_fclose(out);
             break;
         case efPDB:
         case efBRK:
         case efENT:
             out = gmx_fio_fopen(outfile, "w");
-            write_pdbfile(out, title, atoms, x, ePBC, box, ' ', -1, NULL, TRUE);
+            write_pdbfile(out, title, atoms, x, ePBC, box, ' ', -1, nullptr, TRUE);
             gmx_fio_fclose(out);
             break;
         case efESP:
             out = gmx_fio_fopen(outfile, "w");
-            write_espresso_conf_indexed(out, title, atoms, atoms->nr, NULL, x, v, box);
+            write_espresso_conf_indexed(out, title, atoms, atoms->nr, nullptr, x, v, box);
             gmx_fio_fclose(out);
             break;
         case efTPR:
@@ -218,14 +214,12 @@ static void get_stx_coordnum(const char *infile, int *natoms)
         case efG96:
         {
             in        = gmx_fio_fopen(infile, "r");
-            fr.title  = NULL;
             fr.natoms = -1;
-            fr.atoms  = NULL;
-            fr.x      = NULL;
-            fr.v      = NULL;
-            fr.f      = NULL;
-            *natoms   = read_g96_conf(in, infile, &fr, NULL, g96_line);
-            sfree(const_cast<char *>(fr.title));
+            fr.atoms  = nullptr;
+            fr.x      = nullptr;
+            fr.v      = nullptr;
+            fr.f      = nullptr;
+            *natoms   = read_g96_conf(in, infile, nullptr, &fr, nullptr, g96_line);
             gmx_fio_fclose(in);
             break;
         }
@@ -304,7 +298,8 @@ static void tpx_make_chain_identifiers(t_atoms *atoms, t_block *mols)
     }
 }
 
-static void read_stx_conf(const char *infile, t_topology *top,
+static void read_stx_conf(const char *infile,
+                          t_symtab *symtab, char **name, t_atoms *atoms,
                           rvec x[], rvec *v, int *ePBC, matrix box)
 {
     FILE       *in;
@@ -312,11 +307,11 @@ static void read_stx_conf(const char *infile, t_topology *top,
     int         ftp;
     char        g96_line[STRLEN+1];
 
-    if (top->atoms.nr == 0)
+    if (atoms->nr == 0)
     {
         fprintf(stderr, "Warning: Number of atoms in %s is 0\n", infile);
     }
-    else if (top->atoms.atom == NULL)
+    else if (atoms->atom == nullptr)
     {
         gmx_mem("Uninitialized array atom");
     }
@@ -330,72 +325,79 @@ static void read_stx_conf(const char *infile, t_topology *top,
     switch (ftp)
     {
         case efGRO:
-            gmx_gro_read_conf(infile, top, x, v, box);
+            gmx_gro_read_conf(infile, symtab, name, atoms, x, v, box);
             break;
         case efG96:
-            fr.title  = NULL;
-            fr.natoms = top->atoms.nr;
-            fr.atoms  = &top->atoms;
+            fr.natoms = atoms->nr;
+            fr.atoms  = atoms;
             fr.x      = x;
             fr.v      = v;
-            fr.f      = NULL;
+            fr.f      = nullptr;
             in        = gmx_fio_fopen(infile, "r");
-            read_g96_conf(in, infile, &fr, &top->symtab, g96_line);
+            read_g96_conf(in, infile, name, &fr, symtab, g96_line);
             gmx_fio_fclose(in);
             copy_mat(fr.box, box);
-            top->name = put_symtab(&top->symtab, fr.title);
-            sfree(const_cast<char *>(fr.title));
             break;
         case efPDB:
         case efBRK:
         case efENT:
-            gmx_pdb_read_conf(infile, top, x, ePBC, box);
+            gmx_pdb_read_conf(infile, symtab, name, atoms, x, ePBC, box);
             break;
         case efESP:
-            gmx_espresso_read_conf(infile, top, x, v, box);
+            gmx_espresso_read_conf(infile, symtab, name, atoms, x, v, box);
             break;
         default:
             gmx_incons("Not supported in read_stx_conf");
     }
 }
 
-static void done_gmx_groups_t(gmx_groups_t *g)
+static void readConfAndAtoms(const char *infile,
+                             t_symtab *symtab, char **name, t_atoms *atoms,
+                             int *ePBC,
+                             rvec **x, rvec **v, matrix box)
 {
-    int i;
+    int natoms;
+    get_stx_coordnum(infile, &natoms);
 
-    for (i = 0; (i < egcNR); i++)
+    init_t_atoms(atoms, natoms, (fn2ftp(infile) == efPDB));
+
+    bool xIsNull = false;
+    if (x == nullptr)
     {
-        if (NULL != g->grps[i].nm_ind)
-        {
-            sfree(g->grps[i].nm_ind);
-            g->grps[i].nm_ind = NULL;
-        }
-        if (NULL != g->grpnr[i])
-        {
-            sfree(g->grpnr[i]);
-            g->grpnr[i] = NULL;
-        }
+        snew(x, 1);
+        xIsNull = true;
+    }
+    snew(*x, natoms);
+    if (v)
+    {
+        snew(*v, natoms);
+    }
+    read_stx_conf(infile,
+                  symtab, name, atoms,
+                  *x, (v == nullptr) ? nullptr : *v, ePBC, box);
+    if (xIsNull)
+    {
+        sfree(*x);
+        sfree(x);
     }
-    /* The contents of this array is in symtab, don't free it here */
-    sfree(g->grpname);
 }
 
-gmx_bool read_tps_conf(const char *infile, t_topology *top, int *ePBC,
-                       rvec **x, rvec **v, matrix box, gmx_bool bMass)
+void readConfAndTopology(const char *infile,
+                         bool *haveTopology, gmx_mtop_t *mtop,
+                         int *ePBC,
+                         rvec **x, rvec **v, matrix box)
 {
-    t_tpxheader      header;
-    int              natoms, i;
-    gmx_bool         bTop, bXNULL = FALSE;
-    gmx_mtop_t      *mtop;
-    gmx_atomprop_t   aps;
+    GMX_RELEASE_ASSERT(mtop != nullptr, "readConfAndTopology requires mtop!=NULL");
 
-    bTop  = fn2bTPX(infile);
-    if (ePBC != NULL)
+    if (ePBC != nullptr)
     {
         *ePBC = -1;
     }
-    if (bTop)
+
+    *haveTopology = fn2bTPX(infile);
+    if (*haveTopology)
     {
+        t_tpxheader header;
         read_tpxheader(infile, &header, TRUE);
         if (x)
         {
@@ -405,64 +407,55 @@ gmx_bool read_tps_conf(const char *infile, t_topology *top, int *ePBC,
         {
             snew(*v, header.natoms);
         }
-        snew(mtop, 1);
+        int natoms;
         int ePBC_tmp
-            = read_tpx(infile, NULL, box, &natoms,
-                       (x == NULL) ? NULL : *x, (v == NULL) ? NULL : *v, mtop);
-        if (ePBC != NULL)
+            = read_tpx(infile, nullptr, box, &natoms,
+                       (x == nullptr) ? nullptr : *x, (v == nullptr) ? nullptr : *v, mtop);
+        if (ePBC != nullptr)
         {
             *ePBC = ePBC_tmp;
         }
-        *top = gmx_mtop_t_to_t_topology(mtop);
-        /* In this case we need to throw away the group data too */
-        done_gmx_groups_t(&mtop->groups);
-        sfree(mtop);
-        tpx_make_chain_identifiers(&top->atoms, &top->mols);
     }
     else
     {
-        open_symtab(&top->symtab);
-        get_stx_coordnum(infile, &natoms);
-        init_t_atoms(&top->atoms, natoms, (fn2ftp(infile) == efPDB));
-        if (x == NULL)
+        t_symtab   symtab;
+        char      *name;
+        t_atoms    atoms;
+
+        open_symtab(&symtab);
+
+        readConfAndAtoms(infile, &symtab, &name, &atoms, ePBC, x, v, box);
+
+        init_mtop(mtop);
+        convertAtomsToMtop(&symtab, put_symtab(&symtab, name), &atoms, mtop);
+        sfree(name);
+    }
+}
+
+gmx_bool read_tps_conf(const char *infile, t_topology *top, int *ePBC,
+                       rvec **x, rvec **v, matrix box, gmx_bool requireMasses)
+{
+    bool        haveTopology;
+    gmx_mtop_t *mtop;
+
+    // Note: We should have an initializer instead of relying on snew
+    snew(mtop, 1);
+    readConfAndTopology(infile, &haveTopology, mtop, ePBC, x, v, box);
+
+    *top = gmx_mtop_t_to_t_topology(mtop, true);
+    sfree(mtop);
+
+    tpx_make_chain_identifiers(&top->atoms, &top->mols);
+
+    if (requireMasses && !top->atoms.haveMass)
+    {
+        atomsSetMassesBasedOnNames(&top->atoms, TRUE);
+
+        if (!top->atoms.haveMass)
         {
-            snew(x, 1);
-            bXNULL = TRUE;
+            gmx_fatal(FARGS, "Masses were requested, but for some atom(s) masses could not be found in the database. Use a tpr file as input, if possible, or add these atoms to the mass database.");
         }
-        snew(*x, natoms);
-        if (v)
-        {
-            snew(*v, natoms);
-        }
-        read_stx_conf(infile, top, *x, (v == NULL) ? NULL : *v, ePBC, box);
-        if (bXNULL)
-        {
-            sfree(*x);
-            sfree(x);
-        }
-        if (bMass)
-        {
-            aps = gmx_atomprop_init();
-            for (i = 0; (i < natoms); i++)
-            {
-                if (!gmx_atomprop_query(aps, epropMass,
-                                        *top->atoms.resinfo[top->atoms.atom[i].resind].name,
-                                        *top->atoms.atomname[i],
-                                        &(top->atoms.atom[i].m)))
-                {
-                    if (debug)
-                    {
-                        fprintf(debug, "Can not find mass for atom %s %d %s, setting to 1\n",
-                                *top->atoms.resinfo[top->atoms.atom[i].resind].name,
-                                top->atoms.resinfo[top->atoms.atom[i].resind].nr,
-                                *top->atoms.atomname[i]);
-                    }
-                }
-            }
-            gmx_atomprop_destroy(aps);
-        }
-        top->idef.ntypes = -1;
     }
 
-    return bTop;
+    return haveTopology;
 }
