@@ -3,7 +3,7 @@
  *
  * Copyright (c) 1991-2000, University of Groningen, The Netherlands.
  * Copyright (c) 2001-2004, The GROMACS development team.
- * Copyright (c) 2013,2014, by the GROMACS development team, led by
+ * Copyright (c) 2013,2014,2015,2016,2017, by the GROMACS development team, led by
  * Mark Abraham, David van der Spoel, Berk Hess, and Erik Lindahl,
  * and including many others, as listed in the AUTHORS file in the
  * top-level source directory and at http://www.gromacs.org.
@@ -37,17 +37,14 @@
 #ifndef GMX_FILEIO_ENXIO_H
 #define GMX_FILEIO_ENXIO_H
 
-#include "gromacs/fileio/gmxfio.h"
 #include "gromacs/fileio/xdr_datatype.h"
-#include "gromacs/legacyheaders/types/energy.h"
-#include "gromacs/legacyheaders/types/inputrec.h"
-#include "gromacs/legacyheaders/types/state.h"
-
-#ifdef __cplusplus
-extern "C" {
-#endif
+#include "gromacs/trajectory/energy.h"
+#include "gromacs/utility/basedefinitions.h"
 
 struct gmx_groups_t;
+struct t_fileio;
+struct t_inputrec;
+class t_state;
 
 /**************************************************************
  * These are the base datatypes + functions for reading and
@@ -84,6 +81,8 @@ enum {
     enxDHHIST, /* BAR histogram                                              */
     enxDH,     /* BAR raw delta H data                                       */
 
+    enxAWH,    /* AWH data */
+
     enxNR      /* Total number of extra blocks in the current code,
                 * note that the enxio code can read files written by
                 * future code which contain more blocks.
@@ -96,7 +95,7 @@ extern const char *enx_block_id_name[];
 
 /* the subblocks that are contained in energy file blocks. Each of these
    has a number of values of a single data type in a .edr file. */
-typedef struct
+struct t_enxsubblock
 {
     int          nr;        /* number of items in subblock */
     xdr_datatype type;      /* the block type */
@@ -117,7 +116,7 @@ typedef struct
     int lval_alloc;
     int cval_alloc;
     int sval_alloc;
-} t_enxsubblock;
+};
 
 
 /* the energy file blocks. Each block contains a number of sub-blocks
@@ -131,7 +130,7 @@ typedef struct t_enxblock{
 
 
 /* The frames that are read/written */
-typedef struct {
+struct t_enxframe {
     double          t;            /* Timestamp of this frame	                     */
     gmx_int64_t     step;         /* MD step	                             */
     gmx_int64_t     nsteps;       /* The number of steps between frames            */
@@ -144,7 +143,7 @@ typedef struct {
     int             nblock;       /* Number of following energy blocks             */
     t_enxblock     *block;        /* The blocks                                    */
     int             nblock_alloc; /* The number of blocks allocated                */
-} t_enxframe;
+};
 
 /* file handle */
 typedef struct ener_file *ener_file_t;
@@ -175,10 +174,14 @@ void free_enxframe(t_enxframe *ef);
 
 ener_file_t open_enx(const char *fn, const char *mode);
 
-t_fileio *enx_file_pointer(const ener_file_t ef);
+struct t_fileio *enx_file_pointer(const ener_file_t ef);
 
+/* Free the contents of ef */
 void close_enx(ener_file_t ef);
 
+/* Free the contents of ef, and ef itself */
+void done_ener_file(ener_file_t ef);
+
 void do_enxnms(ener_file_t ef, int *nre, gmx_enxnm_t **enms);
 
 void free_enxnms(int n, gmx_enxnm_t *nms);
@@ -188,7 +191,7 @@ gmx_bool do_enx(ener_file_t ef, t_enxframe *fr);
 /* Reads enx_frames, memory in fr is (re)allocated if necessary */
 
 void get_enx_state(const char *fn, real t,
-                   struct gmx_groups_t *groups, t_inputrec *ir,
+                   const gmx_groups_t *groups, t_inputrec *ir,
                    t_state *state);
 /*
  * Reads state variables from enx file fn at time t.
@@ -212,10 +215,8 @@ t_enxblock *find_block_id_enxframe(t_enxframe *ef, int id, t_enxblock *prev);
    subbblocks. */
 void add_subblocks_enxblock(t_enxblock *eb, int n);
 
-
-
-#ifdef __cplusplus
-}
-#endif
+void comp_enx(const char *fn1, const char *fn2, real ftol, real abstol,
+              const char *lastener);
+/* Compare two binary energy files */
 
 #endif
